import sys
import json
import pickle
import re

def read_file_lines(filepath):
    with open(filepath, "r") as f:
        lines = f.readlines()
    return lines


class Node:
    def __init__(self, ipAddress, server, password):
        self.ipAddress = ipAddress
        self.server = server
        self.password = password
        
    def get_ipAddress(self):
        return self.ipAddress
    
    def get_server(self):
        return self.server
    
    def get_password(self):
        return self.password
        


class FCICluster:
    
    def __init__(self, cluster_blueprint_filepath, cluster_template_filepath, host_prop_filepath, cluster_prop_filepath):
        '''
        Input Parameters:
        cluster_blueprint_filepath :    path + file string to cluster blueprint
        cluster_template_filepath :     path + file string to cluster template
        host_prop_filepath :    path + file string to host properties file - fci-hadoop.*.hosts.properties
        cluster_prop_filepath : path + file string to cluster properties file - fci-hadoop.properties
        '''
        
        # Different types of host groups
        self.ambari = None
        self.master = []
        self.secondary = []
        self.gateway = []
        self.slave = []
        self.host_groups = []
        
        # Topology type - supported are dev and prod
        self.topoType = None
        
        with open(cluster_blueprint_filepath, 'r') as file:
            self.cluster_blueprint = json.load(file)
        
        with open(cluster_template_filepath, 'r') as file:
            self.cluster_template = json.load(file)
        
        self.host_properties = read_file_lines(host_prop_filepath)
        self.cluster_properties = read_file_lines(cluster_prop_filepath)
        
        self.setTopologyType()
        self.defineNodes()
        self.setHostGroups()
        self.updateClusterTemplate()
        self.writeUpdatedJson(cluster_template_filepath, self.cluster_template)
        
        if self.topoType == "prod":
            self.updateBlueprint()
            self.writeUpdatedJson(cluster_blueprint_filepath, self.cluster_blueprint)
    
    def setTopologyType(self):
        # Look for: fci.environment.type
        for line in self.cluster_properties:
            if line[0] != "#":
                l = re.split('\s+', line)
                if l[0] == "fci.environment.type":
                    self.topoType = l[2]
                    break
                
    def setHostGroups(self):
        '''
        This function defines the host groups for the cluster depending on topology type
        '''
        if self.topoType == "prod":
            self.host_groups = [["host_group_1", "gateway"], ["host_group_2", "master"], ["host_group_3", "secondary"], ["host_group_4", "slave"]]
        else:
            self.host_groups = [["host_group_1", "master"], ["host_group_2", "gateway"]]
    
    
    def defineNodes(self):
        '''
        This function defines the properties of each server as a Node object and then adds that node to the
        corresponding host group for the topology
        '''
        for line in self.host_properties:
            if line[0] == "#": continue     # Skip lines if they are comments
            
            l = re.split('\s+', line.strip())
            
            if "ambari" in l[0]:
                self.ambari = Node(l[1], l[2], l[3])
            elif "hadoop.gateway" in l[0]:
                self.gateway.append(Node(l[1], l[2], l[3]))
            elif "hadoop.master" in l[0]:
                self.master.append(Node(l[1], l[2], l[3]))
            elif "hadoop.secondary" in l[0]:
                self.secondary.append(Node(l[1], l[2], l[3]))
            elif "hadoop.slave" in l[0]:
                self.slave.append(Node(l[1], l[2], l[3]))
    
    def updateClusterTemplate(self):
        '''
        This function replaces values in the cluster template with values from the hosts properties files. This includes:
        - updating the dummy ambari.server with the actual ambari server
        - adding servers to the correct host groups
        '''
        
        # Replace dummy ambari values
        for config in self.cluster_template["configurations"]:
            if "admin-properties" in config.keys():
                config["admin-properties"]["properties"]["db_host"] = self.ambari.get_server()
            elif "kms-properties" in config.keys():
                config["kms-properties"]["properties"]["db_host"] = self.ambari.get_server()
            elif "dbks-site" in config.keys():
                config["dbks-site"]["properties"]["ranger.ks.jpa.jdbc.url"] = config["dbks-site"]["properties"]["ranger.ks.jpa.jdbc.url"].replace("ambari.server", self.ambari.get_server())
        
        # Add each server to its correct host group
        key_name = "hosts"
        for group in self.host_groups:
            servers = self.getHostGroupServers(group[1])
            
            fqdn = [{"fqdn" : node} for node in servers]
            
            for config in self.cluster_template["host_groups"]:
                if config["name"] == group[0]:
                    config[key_name] = [{"fqdn" : node} for node in servers]
    
    def updateBlueprint(self):
        '''
        This function is responsible for updating the blueprint based on the topology and server information from
        the hosts properties file. Specifically it:
        - Updates the cardinality value for the slave host group
        '''
        cardinality = len(self.slave)
        for group in self.host_groups:
            if group[1] == "slave":
                group_name = group[0]
                break
        
        for group in self.cluster_blueprint["host_groups"]:
            if group["name"] == group_name:
                group["cardinality"] = str(cardinality)
                break
    
    def writeUpdatedJson(self, filepath, json_object):
        '''
        Writes the updated JSON object back out and replaces the original file
        '''
        with open(filepath, "w") as file:
            json.dump(json_object, file, indent=2)
        
    def get_template(self):
        return self.cluster_template
        
    def get_node_count(self):
        count = 0
        if self.ambari != None: count = count + 1  # +1 for ambari node
        count = count + len(self.master) + len(self.slave) + len(self.gateway) + len(self.secondary)    # Add count for master and slave nodes
        return count
    
    def getHostGroupServers(self, group):
        if group == "ambari":
            return [self.ambari.get_server()]
        elif group == "master":
            return [node.get_server() for node in self.master]
        elif group == "gateway":
            return [node.get_server() for node in self.gateway]
        elif group == "secondary":
            return [node.get_server() for node in self.secondary]
        elif group == "slave":
            return [node.get_server() for node in self.slave]
    
    def getHostGroupIPAddress(self, group):
        if group == "ambari":
            return [self.ambari.get_ipAddress()]
        elif group == "master":
            return [node.get_ipAddress() for node in self.master]
        elif group == "gateway":
            return [node.get_ipAddress() for node in self.gateway]
        elif group == "secondary":
            return [node.get_ipAddress() for node in self.secondary]
        elif group == "slave":
            return [node.get_ipAddress() for node in self.slave]
        
    def getAllClusterServers(self):
        servers = []
        for group in self.host_groups:
            servers.extend(self.getHostGroupServers(group[1]))
        return servers
    
    def getAllClusterIpAddresses(self):
        ipaddress = []
        for group in self.host_groups:
            ipaddress.extend(self.getHostGroupIPAddress(group[1]))
        return ipaddress
    

    def getTopologyType(self):
        return self.topoType
    
    def getBlueprintServiceHostServers(self, service):
        service_hosts = []
        host_groups = self.cluster_blueprint["host_groups"]
        for group in host_groups:
            components = group["components"]
            for component in components:
                if component["name"] == service:
                    service_hosts.append(group["name"])
                    break
        
        host_servers = []
        for host in self.host_groups:
            if host[0] in service_hosts:
                host_servers.extend(self.getHostGroupServers(host[1]))
        
        return host_servers
    
    def checkHostGroupAssignment(self):
        total_server_count = self.get_node_count()
        host_server_set = set([self.ambari]) | set(self.master) | set(self.gateway) | set(self.slave) | set(self.secondary)
        unique_server_count = len(host_server_set)
        
        if total_server_count == unique_server_count:
            return "PASS"
        else:
            return "FAIL. Total Server Count is {}, but unique server count is {}".format(total_server_count, unique_server_count)
    
    def test(self):
        return "This is a test message for the {} topology FCI cluster python object".format(self.topoType)
        

if __name__ == '__main__':
    args = sys.argv
    cluster_blueprint_file = args[1]
    cluster_template_file = args[2] # cluster template - path + file
    host_prop_file = args[3]    # host properties file (*.hosts.properties) - path + file
    cluster_prop_file = args[4] # cluster properties file (*.properties) - path + file
    
    fci_cluster = FCICluster(cluster_blueprint_file, cluster_template_file, host_prop_file, cluster_prop_file)
    
    
    with open("fci_cluster.pkl", "wb") as file:
      pickle.dump(fci_cluster, file)
    
    
    
