#!/bin/sh
##################################################################
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
#
# (C) Copyright IBM Corp. 2017, 2018
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
##################################################################

. gettext.sh

# Check that script is run as root user
if [ "$EUID" -ne 0 ]; then
	log.error "Please run this script as user root."
	exit
fi

# Capture the current working directory
SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)
SCRIPT_NAME=$(basename "$0")

# Define variables to create a lock on the script
LOCKFILE="${SCRIPT_DIR}/$SCRIPT_NAME.lock"
LOCKFD=99

# Lock functions
_lock()				{ echo $$>$LOCKFILE; flock -$1 $LOCKFD; }
_remove_lock()		{ _lock u; _lock xn && rm -f $LOCKFILE; }
_init_lock()		{ eval "exec $LOCKFD>\"$LOCKFILE\""; trap _remove_lock EXIT; }
_exclusivelock()	{ _lock xn; }
_alreadylocked() 	{ printf "\nUnable to attain an exclusive lock on $SCRIPT_DIR/`basename $0`.  Exiting.\nPlease verify that this script isn't already running.\n\n"; exit 1; }

# Create lock
_init_lock
_exclusivelock || _alreadylocked

# Timestamp
timestamp () {
	date +"%F_%T"
}

# Set aliases
alias log.info='echo [$(timestamp)][INFO][line $LINENO] ${INSTALL_STEP} - '
alias log.error='echo [$(timestamp)][ERROR][line $LINENO] ${INSTALL_STEP} - '
alias log.warn='echo [$(timestamp)][WARNING][line $LINENO] ${INSTALL_STEP} - '

# Define variables
typeset -i rc=0 list_size=0
typeset -a PREREQ_LIST repolist

# Set variables
HDP_VERSION="2.6.4.0-91"
AMBARI_VERSION="2.6.1.5-3"
VERSION="1.0.3"
SSHKEY_NAME=fciambari_rsa

# Set python script variables and names
CREATE_FCI_CLUSTER_PY=create_fci_cluster_object.py
READ_FCI_CLUSTER_PY=read_fci_cluster_object.py

# License file
LICENSE=L-KSEL-AWPRJ5
LICENSE_FORMAT=zip

# Define properties files
PROPERTY_FILE=fci-hadoop.properties

# Required media files
AMBARI_REPO=ambari-2.6.1.5-centos7.tar.gz
HDP_REPO=HDP-2.6.4.0-centos7-rpm.tar.gz
HDP_UTILS_REPO=HDP-UTILS-1.1.0.22-centos7.tar.gz
HDP_GPL_REPO=HDP-GPL-2.6.4.0-centos7-rpm.tar.gz
MYSQL_LIB=mysql-connector-java-5.1.46.tar.gz

# Required rpms
TEXINFO_RPM=texinfo-tex-5.1-4.el7.x86_64.rpm
TEXTLIVE_RPM=texlive-epsf-svn21461.2.7.4-38.el7.noarch.rpm
TEXTLIVE_DOC_RPM=texlive-epsf-doc-svn21461.2.7.4-38.el7.noarch.rpm

# Create lock directory
LOCK_DIR="${SCRIPT_DIR}/.${VERSION}.locks/"
mkdir -p ${LOCK_DIR}

##################################################################

# Define function
function getPropertyValue() {
	local value=""
	local rc=0
	while true
	do
		[[ "$#" -ne 2 ]] && rc=1 && break
		typeset -r propertyFile=$1
		local property=$2
		property="${property%%[[:space:]]}"
		property="${property##[[:space:]]}"
		[[ "${property}" == "" ]] && rc=2 && break
		[[ ! -f "${propertyFile}" ]] && rc=3 && break
		value=$(sed '/^\#/d' "${propertyFile}" | grep "^[[:space:]]*${property}[=: ]" | tail -n 1 )
		[[ "${value}" == "" ]] && rc=4 && break
		# Remove leading/trailing white space
		value="${value%%[[:space:]]}"
		value="${value##[[:space:]]}"
		# Isolate value (split on ':' or '=')
		value=$(echo "${value}" | sed 's/[^:=]*//')
		# delete everything after the first # (comment)
		value=${value%%"#"*}
		# Remove leading value whitespace
		value=$(echo "${value}" | sed 's/^[:=][[:space:]]*//')
		# Remove "Windows" CR character (if any) from value.
		value=$(echo "${value}" | sed 's|\r$||')
		break
	done
	echo -n "${value}"
	return $rc
}

promptForLicenseIntructions() {
	echo; echo
	echo "In order to continue the installation, the license will need to reviewed and accepted."
	echo "When viewing the license, you can navigate using common VIM commands."
	echo
	echo "   Common VIM commands include:"
	echo
	echo "       k - Moves the cursor up one line"
	echo "       j - Moves the cursor down one line"
	echo "      ^F (CTRL F) - Scrolls forward one screen"
	echo "      ^B (CTRL B) - Scrolls backward one screen"
	echo "     :q! - Exit the license acceptance viewer"
	echo
	echo "Once the license has been reviewed, exit the license viewer by typing ':quit or :q!' to exit."
	read -s -p "Press [Enter] key to view the license..."
	echo
}

function findLicenseDir() {
	local target="license/$LICENSE"
	local dir="$SCRIPT_DIR"

	while [[ -e "$dir" ]]; do
		if [[ -e "$dir/$target" ]]; then
			echo -n "$dir/$target"
			return 0
		fi
		dir="${dir%/*}"
	done
	return 1
}

function findLicenseFileFromVar() {
	local -r license_dir="$1"; shift
	local -r prefix="$1"; shift
	local suffix="$1"; shift
	local file="$license_dir/$prefix$suffix"

	# lang_region.encoding
	[[ -e "$file" ]] && { echo -n "$file"; return 0; }

	# lang_region
	suffix=${suffix%\.*}
	file="$license_dir/$prefix$suffix"
	[[ -e "$file" ]] && { echo -n "$file"; return 0; }

	# lang
	suffix=${suffix%_*}
	file="$license_dir/$prefix$suffix"
	[[ -e "$file" ]] && { echo -n "$file"; return 0; }

	return 1
}

findLicenseFile() {
	local -r prefix="$1"; shift
	local -r license_dir=$(findLicenseDir)
	local -r -i license_dir_ok=$?

	[[ $license_dir_ok -eq 0 ]] || { log.error "Unable to locate the directory with the license files." && echo; return 1; }

	local license_file=$(findLicenseFileFromVar "$license_dir" "$prefix" "$LC_ALL")
	[[ $license_file != "" ]] && { echo -n "$license_file"; return 0; }

	license_file=$(findLicenseFileFromVar "$license_dir" "$prefix" "$LANG")
	[[ $license_file != "" ]] && { echo -n "$license_file"; return 0; }

	license_file=$(findLicenseFileFromVar "$license_dir" "$prefix" "en_US")
	[[ $license_file != "" ]] && { echo -n "$license_file"; return 0; }

	return 1
}

promptForLicenseAcceptance() {
	local -r license_file="${1}"; shift
	local vi_cmd=$(command -v vi)
	local -r yes=y
	local -r no=n
	local rc=1

	[[ $vi_cmd != "" ]] || { log.error "Unable to display the license file, because no viewer application was found."; return 1; }

	"$vi_cmd" "$license_file"
	echo; echo
	eval_gettext "Do you accept the terms of this license?  ($yes/$no): "
	while true; do
		read response
		[[ ${response} == "$yes" ]] && { rc=0; break; }
		[[ ${response} == "$no" ]] && { break; }
		echo
		eval_gettext "The response \"$response\" is not valid.  Please enter $yes or $no: "
	done

	return $rc
}

acceptLicenseAgreement_FCI() {
	INSTALL_STEP="Accept license agreement"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.acceptLicenseAgreement_FCI.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping acceptLicenseAgreement_FCI step. `cat ${lockfile}`" && return $error_count; fi

	local -r license_file_LA=$(findLicenseFile LA_)
	local -r -i license_file_LA_rc=$?

	local -r license_file_LI=$(findLicenseFile LI_)
	local -r -i license_file_LI_rc=$?

	[[ $license_file_LA_rc -eq 0 ]] && [[ $license_file_LI_rc -eq 0 ]] || { echo "$license_file"; return 1; }

	license_file=/tmp/license.txt
	cat $license_file_LA > $license_file
	cat $license_file_LI >> $license_file

	promptForLicenseAcceptance "$license_file"
	[[ $? -eq 0 ]] || error_count=$((error_count + 1))

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function determineHostPropertiesFile() {
	log.info "=================================================================================="
	log.info "Checking for ${PROPERTY_FILE} file."
	log.info "=================================================================================="
	if [[ ! -f "$SCRIPT_DIR/$PROPERTY_FILE" ]]; then
		log.error "The properties file could not be found. Verify that ${PROPERTY_FILE} exists."
		error_count=$((error_count + 1))
		exit $error_count
	fi
	log.info "${PROPERTY_FILE} has been found."
	
	# Determine the installation type
	ENV_TYPE=$(getPropertyValue "${PROPERTY_FILE}" "fci.environment.type")

	INSTALL_STEP="Validate Topology"
	log.info "=================================================================================="
	log.info "Setting hosts properties file to use."
	log.info "=================================================================================="
	if [ -z "$ENV_TYPE" ]; then
		log.error "The environment type could not be determined. Verify the fci.environment.type is set to dev or prod in ${PROPERTY_FILE}."
		error_count=$((error_count + 1))
		exit $error_count
	else
		HOSTNAME_FILE=fci-hadoop.${ENV_TYPE}.hosts.properties
	fi
	log.info "Using $HOSTNAME_FILE hosts properties file."
	
	
	log.info "=================================================================================="
	log.info "Checking for ${HOSTNAME_FILE} file."
	log.info "=================================================================================="
	if [[ ! -f "$SCRIPT_DIR/$HOSTNAME_FILE" ]]; then
		log.error "The host properties file could not be found. Verify that ${HOSTNAME_FILE} exists."
		error_count=$((error_count + 1))
		exit $error_count
	fi
	log.info "${HOSTNAME_FILE} has been found."

	# Remove blank lines from properties file
	sed -i -e '/^$/d' $HOSTNAME_FILE
	
	CONFIG_DIR=config/hadoop-cluster
	CLUSTER_TEMPLATE_NAME=fci-hadoop-${ENV_TYPE}-cluster-template.json
	CLUSTER_BLUEPRINT_NAME=fci-hadoop-${ENV_TYPE}-blueprint-hdp-2.6.4.json
	
	log.info "=================================================================================="
	log.info "Checking for correct blueprint - $CLUSTER_BLUEPRINT_NAME - and cluster template - $CLUSTER_TEMPLATE_NAME"
	log.info "=================================================================================="
	
	if [[ -f "$SCRIPT_DIR/$CONFIG_DIR/$CLUSTER_BLUEPRINT_NAME" ]]; then
		log.info "The blueprint file $CLUSTER_BLUEPRINT_NAME exists."
	else
		log.error "The blueprint $CLUSTER_BLUEPRINT_NAME does not exist"
		error_count=$((error_count + 1))
		exit $error_count
	fi
	if [[ -f "$SCRIPT_DIR/$CONFIG_DIR/$CLUSTER_TEMPLATE_NAME" ]]; then
		log.info "The cluster template file $CLUSTER_TEMPLATE_NAME exists."
	else
		log.error "The cluster template file $CLUSTER_TEMPLATE_NAME does not exist"
		error_count=$((error_count + 1))
		exit $error_count
	fi

	return $error_count
}

function validateHostTypes() {
	local error_count=0 ENV_SIZE host_entries lockfile="${LOCK_DIR}/.${VERSION}.validateHostTypes.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping validateHostTypes step. `cat ${lockfile}`" && return $error_count; fi

	# Validate host types in HOSTNAME_FILE
	INSTALL_STEP="Validating host types in $HOSTNAME_FILE"
	log.info "=================================================================================="
	log.info "Validating host types."
	log.info "=================================================================================="
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
        # Check that all host types are valid
		if [[ $HOST_TYPE != ambari ]] && [[ $HOST_TYPE != hadoop* ]] && [[ $HOST_TYPE != kube.master ]] && [[ $HOST_TYPE != kube.worker* ]] ; then
    		log.warn "Unable to process all entries in the $HOSTNAME_FILE file."
    		log.error "The $HOST_TYPE is not valid."
    		log.info "Verify that the host type labels specified are either ambari, hadoop*, kube.master, or kuber.worker*."
 			error_count=$((error_count + 1))
 		else
 			log.info "$HOSTNAME_FILE - $HOST_TYPE is valid. "
 		fi
	done < $HOSTNAME_FILE
	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function validateInstallComponents() {
	log.info "=================================================================================="
	log.info "Validating Install Components."
	log.info "==================================================================================" && echo
	
	# This function is used to validate anything we want to in the blueprint, cluster template, and *.hosts.properties files
	checkHostGroupAssignment=$(python ${SCRIPT_DIR}/$READ_FCI_CLUSTER_PY "CHECK HOST ASSIGNMENT") ||  { log.error "Unable to use FCI Cluster Object function checkHostGroupAssignment." && echo; error_count=$((error_count + 1)); return 1; }
	if [[ $checkHostGroupAssignment == "PASS" ]]; then
		log.info "checkHostGroupAssignment check has PASSED!" && echo
	else
		log.error "checkHostGroupAssignment check has FAILED" && echo; error_count=$((error_count + 1));
	fi
}

function createFciClusterObject() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.createFciClusterObject.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping createFciClusterObject step. `cat ${lockfile}`" && return $error_count; fi
	# Install Open JDK 1.8 on all servers
	INSTALL_STEP="Create FCI Object"
	
	log.info "=================================================================================="
	log.info "Creating FCI Cluster Object."
	log.info "==================================================================================" && echo
	
	# Create the python object
	python ${SCRIPT_DIR}/$CREATE_FCI_CLUSTER_PY ${SCRIPT_DIR}/${CONFIG_DIR}/$CLUSTER_BLUEPRINT_NAME ${SCRIPT_DIR}/${CONFIG_DIR}/$CLUSTER_TEMPLATE_NAME ${SCRIPT_DIR}/$HOSTNAME_FILE ${SCRIPT_DIR}/$PROPERTY_FILE
	
	# Test using the python object
	msg=$(python ${SCRIPT_DIR}/$READ_FCI_CLUSTER_PY "TEST") ||  { log.error "Unable to load and use FCI Cluster Object." && echo; error_count=$((error_count + 1)); return 1; }
	log.info "$msg"
	
	validateInstallComponents || { log.error "Unable to validate install components." && echo; }
	
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function generatePrereqScript() {
	local rc=0
	# Initialize bash script
	echo "#!/bin/sh" > $prereqScriptPath/$prereqScript

	# Update the script with pre-req steps
	typeset -i list_iter=0
	while [[ $list_iter -lt $list_size ]]
	do
		cat "${PREREQ_LIST[$list_iter]}" >> $prereqScriptPath/$prereqScript
		if [[ $rc -ne 0 ]]; then
			log.error "Terminating due to return code: $rc from ${PREREQ_LIST[$list_iter]}"
			break 2
		fi
		list_iter+=1
		echo "" >> $prereqScriptPath/$prereqScript
	done
	return $rc
}

function processPrereqList_local() {
	# Create prereq script for remote execution
	prereqScriptPath="${SCRIPT_DIR}/prepare/_generatedScripts"
	prereqScript="install_prereqs_$HOST_TYPE.sh"
	generatePrereqScript || { log.error "Generation of pre-requisite script failed." && echo; return 1; }

	# Run $PrereqScript
	chmod u+x $prereqScriptPath/$prereqScript
	$prereqScriptPath/$prereqScript
	rc=$?
	return $rc
}

function processPrereqList_remote() {
	# Create prereq script for remote execution
	prereqScriptPath="${SCRIPT_DIR}/prepare/_generatedScripts"
	prereqScript="install_prereqs_$HOST_TYPE.sh"
	generatePrereqScript || { log.error "Generation of pre-requisite script failed." && echo; return 1; }

	# Upload files to host and run
	scp $prereqScriptPath/$prereqScript root@$HOSTNAME_FQDN:~
	ssh -n root@$HOSTNAME_FQDN "chmod u+x $prereqScript;./$prereqScript; rm $prereqScript"
	rc=$?
	return $rc
}

function precheckFiles() {
	local rc=0 error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.precheckFiles.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping precheckFiles step. `cat ${lockfile}`" && return $error_count; fi
	# Check for required properties files
	INSTALL_STEP="Checking for required properties files"
	log.info "=================================================================================="
	log.info "Checking for $PROPERTY_FILE and $HOSTNAME_FILE in $SCRIPT_DIR."
	log.info "=================================================================================="
	if [[ ! -f "$PROPERTY_FILE" ]]; then
		log.error "Required properties file: ${PROPERTY_FILE} could not be found in $SCRIPT_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $PROPERTY_FILE properties file."
	fi
	if [[ ! -f "$HOSTNAME_FILE" ]]; then
		log.error "Required properties file: ${HOSTNAME_FILE} could not be found in $SCRIPT_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $HOSTNAME_FILE properties file."
	fi

	if [[ ! -f "${SCRIPT_DIR}/media/$AMBARI_REPO" ]]; then
		log.error "Required repository file: ${AMBARI_REPO} could not be found in $SCRIPT_DIR/media/."
		error_count=$((error_count + 1))
	else
		log.info "Found $AMBARI_REPO repository file."
	fi
	if [[ ! -f "${SCRIPT_DIR}/media/$HDP_REPO" ]]; then
		log.error "Required repository file: ${HDP_REPO} could not be found in $SCRIPT_DIR/media/."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_REPO repository file."
	fi
	if [[ ! -f "${SCRIPT_DIR}/media/$HDP_UTILS_REPO" ]]; then
		log.error "Required repository file: ${HDP_UTILS_REPO} could not be found in $SCRIPT_DIR/media/."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_UTILS_REPO repository file."
	fi
	if [[ ! -f "${SCRIPT_DIR}/media/$HDP_GPL_REPO" ]]; then
		log.error "Required repository file: ${HDP_GPL_REPO} could not be found in $SCRIPT_DIR/media/."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_GPL_REPO repository file."
	fi
	if [[ ! -f "${SCRIPT_DIR}/media/$MYSQL_LIB" ]]; then
		log.error "Required repository file: ${MYSQL_LIB} could not be found in $SCRIPT_DIR/media/."
		error_count=$((error_count + 1))
	else
		log.info "Found $MYSQL_LIB library file."
	fi

	if [[ "$error_count" != "0" ]]; then
		log.error "Total number of missing pre-requisite files: $error_count." && echo && return $error_count
	fi
	echo

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function precheckMySQL() {
	local rc=0 error_count=0 list_size=0 lockfile="${LOCK_DIR}/.${VERSION}.precheckMySQL.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping precheckMySQL step. `cat ${lockfile}`" && return $error_count; fi
	INSTALL_STEP="Verify MySQL rpms are installed"
	# Required RPM listed
	unset PREREQ_LIST
	MYSQL_RPM_LIST[list_size++]="mysql-community-server-"
	MYSQL_RPM_LIST[list_size++]="mysql-community-common-"
	MYSQL_RPM_LIST[list_size++]="mysql-community-libs-"
	MYSQL_RPM_LIST[list_size++]="mysql-community-client-"
	MYSQL_RPM_LIST[list_size++]="mysql-community-release-"

	# Determine where MySQL should be installed
	MYSQL_SERVER=$(python ${SCRIPT_DIR}/$READ_FCI_CLUSTER_PY "GET SERVICE_HOST MYSQL_SERVER")
	log.info "MySQL host server is set to $MYSQL_SERVER"

	log.info "=================================================================================="
	log.info "Verifying MySQL is installed on $MYSQL_SERVER."
	log.info "=================================================================================="
	# Check for required RPMs
	typeset -i list_iter=0
	while [[ $list_iter -lt $list_size ]]
	do
		rpm_installed="`ssh root@$MYSQL_SERVER "if rpm -qa | grep -q ${MYSQL_RPM_LIST[$list_iter]}; then echo 'FOUND'; else echo 'NOT FOUND';fi"`"
		#log.info "${MYSQL_RPM_LIST[$list_iter]} $rpm_installed"
		if [[ "$rpm_installed" = "FOUND" ]]; then
	        log.info "${MYSQL_RPM_LIST[$list_iter]} rpm" is installed
	     else
	        log.info "${MYSQL_RPM_LIST[$list_iter]} rpm is NOT INSTALLED"
					log.error "Install ${MYSQL_RPM_LIST[$list_iter]} on $MYSQL_SERVER before continuing installation."
					error_count=$((error_count + 1))
		fi
		list_iter+=1
	done

	if [[ "$error_count" != "0" ]]; then
		log.error "Total number of missing pre-requisites: $error_count." && echo && return $error_count
	else
		log.info "All required pre-requisite MySQL rpms are installed on $MYSQL_SERVER"
	fi
	echo

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function pingTest() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.pingTest.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping pingTest step. `cat ${lockfile}`" && return $error_count; fi
	# Simple ping test (servers listed in fci-hadoop-*.hosts.properties)
	INSTALL_STEP="Ping Test"
	log.info "=================================================================================="
	log.info "Testing ability to ping servers in $HOSTNAME_FILE."
	log.info "=================================================================================="
	
	IP_ADDRESS=($(python ${SCRIPT_DIR}/$READ_FCI_CLUSTER_PY "GET NODE_IPADDRESS ALL" | tr -d '[],'))
	HOSTNAME_FQDN=($(python ${SCRIPT_DIR}/$READ_FCI_CLUSTER_PY "GET NODE_SERVER ALL" | tr -d '[],'))
	
	for index in ${!IP_ADDRESS[@]}
	do
		ping -q -c3  ${IP_ADDRESS[index]} > /dev/null
		if [ $? -eq 0 ]; then
			log.info "Ping connection test to ${IP_ADDRESS[index]} is ok."
		else
			log.error "Ping connection test to ${IP_ADDRESS[index]} failed. Please check network configuration"
			error_count=$((error_count + 1))
		fi

		ping -q -c5  ${HOSTNAME_FQDN[index]} > /dev/null
		if [ $? -eq 0 ]; then
			log.info "Ping connection test to ${HOSTNAME_FQDN[index]} is ok."
		else
			log.error "Connection to ${HOSTNAME_FQDN[index]} failed. Please check network configuration"
			error_count=$((error_count + 1))
		fi
	done
	log.info "=================================================================================="
	echo

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function installExpect() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.installExpect.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping installExpect step. `cat ${lockfile}`" && return $error_count; fi

	# Install expect
	INSTALL_STEP="Installing expect package"
	log.info "=================================================================================="
	log.info "Checking that expect rpm is installed."
	log.info "=================================================================================="
	if  rpm -qa | grep -q expect; then
		log.info "Expect rpm already installed"
		installexpectresult=0
		echo
	else
		log.info "Installing expect rpm via yum"
		echo
		yum -y install expect
		installexpectresult=$?
		echo
	fi

	# If there are no errors, create a lockfile to mark completion
	if [[ $installexpectresult = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $installexpectresult

}

function setupSSHTrust () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.setupSSHTrust.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping setupSSHTrust step. `cat ${lockfile}`" && return $error_count; fi

	# Generate SSH public/private key pair
	INSTALL_STEP="Setting up SSH trust"
	log.info "Removing existing id_rsa key pair."

	# Remove existing id_rsa if it exists
	if [[ -f /root/.ssh/id_rsa ]]; then
		/usr/bin/rm -f /root/.ssh/id_rsa && /usr/bin/rm -f /root/.ssh/id_rsa.pub
	fi

	# Generate a new id_rsa key
	log.info "Generating a new SSH key private/public keypair."
	ssh-keygen -q -t rsa -N "" -f /root/.ssh/id_rsa || { log.error "Creation of private/public keypair failed." && echo; return 1; }

	# Setup trust from ambari/install server to all other servers
	log.info "Uploading public SSH key to servers."
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN ADMIN_PW
	do
		# Only execute on remote hosts
		if [[ $HOSTNAME_FQDN != `hostname -f` ]]; then
			if [[ $HOST_TYPE == ambari ]] || [[ $HOST_TYPE == hadoop* ]] ; then
				log.info "Creating trust from `hostname -f` to $HOST_TYPE server $HOSTNAME_FQDN."

				# Upload public key and update authorized_keys
				log.info "Uploading required files."
				serverpw=$ADMIN_PW
				runExpect 'ssh -q -n' root@$HOSTNAME_FQDN "mkdir -p /root/.ssh/" || { log.error "Unable to create /root/.ssh directory on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
				runExpect 'ssh -q -n' root@$HOSTNAME_FQDN "rm -f /root/.ssh/authorized_keys" || { log.error "Unable to remove /root/.ssh/authorized keys on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
				runExpect scp /root/.ssh/id_rsa.pub root@$HOSTNAME_FQDN:/root/.ssh/${SSHKEY_NAME}.pub || { log.error "Unable to upload id_rsa.pub to $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
				runExpect ssh-copy-id '-i /root/.ssh/id_rsa.pub' root@$HOSTNAME_FQDN || { log.error "Unable to execute ssh-copy-id to remote host $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
			fi
		fi
	done < $HOSTNAME_FILE

	log.info "Setup trust to Ambari server/self."
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN ADMIN_PW
	do
		# Only execute on local host
		if [[ $HOSTNAME_FQDN == `hostname -f` ]]; then
			if [[ $HOST_TYPE == ambari ]] ; then
				log.info "Creating trust from `hostname -f` to $HOST_TYPE server $HOSTNAME_FQDN."

				# Upload public key and update authorized_keys
				log.info "Update authorized keys."
				serverpw=$ADMIN_PW
				runExpect ssh-copy-id '-i /root/.ssh/id_rsa.pub' root@$HOSTNAME_FQDN || { log.error "Unable to execute ssh-copy-id to remote host $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
			fi
		fi
	done < $HOSTNAME_FILE

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function sshTest() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.sshTest.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping sshTest step. `cat ${lockfile}`" && return $error_count; fi
	# Checks if ssh root trusts are enabled (servers listed in fci-hadoop-*.hosts.properties)
	INSTALL_STEP="SSH Test"
	log.info "=================================================================================="
	log.info "Testing ability to ssh servers in $HOSTNAME_FILE."
	log.info "=================================================================================="
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  	if [[ $HOST_TYPE == hadoop* ]] || [[ $HOST_TYPE == db ]] || [[ $HOST_TYPE == elk ]]; then
			sshtest=$(ssh -n -o BatchMode=yes -o ConnectTimeout=5 root@$HOSTNAME_FQDN echo ok 2>&1)
			if $(echo $sshtest | grep --silent 'ok'); then
				log.info "SSH connection test to $HOSTNAME_FQDN is ok."
			else
				log.info "SSH connection test to $HOSTNAME_FQDN FAILED"
				error_count=$((error_count + 1))
			fi
		fi
	done < $HOSTNAME_FILE
	if [[ $error_count -gt 0 ]]; then
		log.error "SSH connection tests failed.  Please check the sshd authorized keys on that target server for issues."
		log.info "Check that the passwords defined in the properties file are accurate."
		log.warn "Error count is: $errorcount"
		return $error_count
	else
		log.info "SSH connection tests were successful."
		echo "Completed `date`">$lockfile
	fi
	log.info "=================================================================================="
	echo
	return $error_count
}

function runExpect() {
	# Uses expect for remote logins before root ssh trust is established
	RUN_EXPECT=$(expect -c "
		log_user 0
		spawn -noecho $1 $2 $3
			expect {
					-re \".*Are you sure you want to continue connecting.*\" { send \"yes\n\"; exp_continue }
						\"password: \" {send -- \"$serverpw\r\";exp_continue}
						eof
					}
		log_user 1")

	echo "$RUN_EXPECT"
	return $rc
}

function checkUMASK () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.checkUMASK.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping checkUMASK step. `cat ${lockfile}`" && return $error_count; fi
	# Check umask setting on all servers
	echo
	INSTALL_STEP="Check umask"
	log.info "Checking server umask setting on all servers."
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
		# Check on remote hosts
		if [[ $HOSTNAME_FQDN != `hostname -f` ]]; then
			if [[ $HOST_TYPE == hadoop* ]] || [[ $HOST_TYPE == db ]] || [[ $HOST_TYPE == elk ]]; then
				CHECK_UMASK=`ssh -n root@$HOSTNAME_FQDN 'umask'`
			fi
		else
			# Check local umask setting
			CHECK_UMASK="`umask`"
		fi

		# Check that umask is set to 0022
		if [[ $CHECK_UMASK != "0022" ]];
		then
			log.info "Umask is set to $CHECK_UMASK on $HOSTNAME_FQDN"
			log.info "Umask setting needs to be 0022"
			log.info "Update umask on $HOSTNAME_FQDN before continuing"
			error_count=$((error_count + 1))
		else
			log.info "Umask is set to $CHECK_UMASK on $HOSTNAME_FQDN"
		fi
	done < $HOSTNAME_FILE
	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function updateOSconfig() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.updateOSconfig.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping updateOSconfig step. `cat ${lockfile}`" && return $error_count; fi
	# Deploy pre-reqs on servers in HOSTNAME_FILE
	INSTALL_STEP="Updating OS configuration"
	# Create required directory
	log.info "Creating _generatedScripts directory"
	mkdir -p ${SCRIPT_DIR}/prepare/_generatedScripts || { log.error "Unable to create ${SCRIPT_DIR}/prepare/_generatedScripts directory." && echo; return 1; }
	log.info "Processing hosts in $HOSTNAME_FILE"
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  if [[ $HOST_TYPE == ambari ]] || [[ $HOST_TYPE == hadoop* ]]; then
		log.info "=================================================================================="
		log.info "Hostname: $HOSTNAME_FQDN - HostType: $HOST_TYPE"
		log.info "=================================================================================="
		echo
		if [[ $HOST_TYPE == hadoop* ]]; then
			list_size=0
			unset PREREQ_LIST
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/install_rpms_hadoop-cluster.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_transparent_hugepages.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_ipv6.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/update_ulimits.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/enable_ntpd.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_selinux.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_fw.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/update_fw_hadoop-cluster.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/cleanup_ambari_agent.txt"
			processPrereqList_remote
			echo
		fi
		if [[ $HOST_TYPE == ambari ]]; then
			list_size=0
			unset PREREQ_LIST
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/install_rpms_ambari.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_ipv6.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/update_ulimits.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/enable_ntpd.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/disable_selinux.txt"
			PREREQ_LIST[list_size++]="${SCRIPT_DIR}/prepare/update_fw_ambari.txt"

			# Check if installing locally
			if [[ $HOSTNAME_FQDN == `hostname -f` ]]; then
				processPrereqList_local && echo ||  { log.error "Unable to complete OS configuration update." && echo; return 1; }
			else
				processPrereqList_remote && echo ||  { log.error "Unable to complete OS configuration update." && echo; return 1; }
			fi
		fi
	  fi
	done < $HOSTNAME_FILE
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function installJDK() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.installJDK.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping installJDK step. `cat ${lockfile}`" && return $error_count; fi
	# Install Open JDK 1.8 on all servers
	INSTALL_STEP="Install Open JDK 1.8"
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
		if [[ $HOST_TYPE == ambari ]] || [[ $HOST_TYPE == hadoop* ]]; then
			if [[ $HOSTNAME_FQDN != `hostname -f` ]]; then
				log.info "=================================================================================="
				log.info "Installing Open JDK 1.8 on $HOSTNAME_FQDN."
				log.info "==================================================================================" && echo
				# Upload files to host and run
				scp ${SCRIPT_DIR}/prepare/install_java_openjdk.txt root@$HOSTNAME_FQDN:~
				ssh -n root@$HOSTNAME_FQDN 'bash install_java_openjdk.txt; rm install_java_openjdk.txt' ||  { log.error "Unable to install Open JDK 1.8 on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); return 1; }
				echo

			fi
			if [[ $HOSTNAME_FQDN == `hostname -f` ]]; then
				log.info "=================================================================================="
				log.info "Installing Open JDK 1.8 locally."
				log.info "==================================================================================" && echo
				bash ${SCRIPT_DIR}/prepare/install_java_openjdk.txt ||  { log.error "Unable to install Open JDK 1.8 on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); return 1; }
			fi
		fi
	done < $HOSTNAME_FILE
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function installHDP () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.installHDP.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping installHDP step. `cat ${lockfile}`" && return $error_count; fi
	INSTALL_STEP="Install Hadoop-Cluster"
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  if [[ $HOST_TYPE == ambari ]] && [[ $HOSTNAME_FQDN == `hostname -f` ]] ; then
		log.info "Starting Installation of Hadoop-Cluster on $HOSTNAME_FQDN"
		log.info "========================================================================"
		log.info "Install Hortonworks Data Platform on $HOSTNAME_FQDN"
		log.info "========================================================================"
		# Extract properties from property file
		#uiPort=$(getPropertyValue "${PROPERTY_FILE}" "AMBARI.SERVER.PORT")
		uiPort=8081
		postgresPWD=$(getPropertyValue "${PROPERTY_FILE}" "postgres.admin.pwd")

		# Additional properties
		installScript=install.hdp.sh
		mediaDir=$SCRIPT_DIR/media/

		# Run HDP Installation
		log.info "Starting Hortonworks Data Platform Installation."
		echo
		sudo chmod u+x ${SCRIPT_DIR}/install/hadoop-cluster/${installScript} ||  { log.error "Unable to update permissions on ${SCRIPT_DIR}/install/hadoop-cluster/${installScript}." && echo; error_count=$((error_count + 1)); return 1; }
		sudo ${SCRIPT_DIR}/install/hadoop-cluster/$installScript $mediaDir $uiPort $postgresPWD $AMBARI_REPO $HDP_REPO $HDP_UTILS_REPO $HDP_GPL_REPO $MYSQL_LIB $AMBARI_VERSION $HDP_VERSION ||  { log.error "Installation of Hortonworks Data Platform failed." && echo; error_count=$((error_count + 1)); return 1; }
	  fi
	done < $HOSTNAME_FILE
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function configureAmbariSSL () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.configureAmbariSSL.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping configureAmbariSSL step. `cat ${lockfile}`" && return $error_count; fi
	INSTALL_STEP="Configure Ambari SSL"
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  if [[ $HOST_TYPE == ambari ]] && [[ $HOSTNAME_FQDN == `hostname -f` ]] ; then
		log.info " ========================================================================"
		log.info " Configure Ambari SSL"
		log.info " ========================================================================"

		# Additional properties
		installScript=config.ssl.sh

		# Enable Ambari SSL configuration
		log.info " Starting Ambari SSL configuration."
		sudo ${SCRIPT_DIR}/install/hadoop-cluster/$installScript $SCRIPT_DIR $HOSTNAME_FILE ||  { log.error "Error configuring Ambari SSL." && echo; error_count=$((error_count + 1)); return 1; }
	  fi
	done < $HOSTNAME_FILE
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}


function deployAmbariAgents () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.deployAmbariAgents.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping deployAmbariAgents step. `cat ${lockfile}`" && return $error_count; fi
	# Register cluster members with Ambari server
	INSTALL_STEP="Add Cluster Members"
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  if [[ $HOST_TYPE == hadoop* ]] ; then
		echo
		log.info "Registering $HOSTNAME_FQDN with Ambari server"
		log.info "========================================================================"
		log.info "Add $HOSTNAME_FQDN as Ambari agent - Host Type:$HOST_TYPE"
		log.info "========================================================================"

		# Additional properties
		installScript=install.agent.sh
		ambariHost=`hostname -f`

		# Install Ambari agent
		log.info "Uploading required files to $HOSTNAME_FQDN." && echo
		scp $SCRIPT_DIR/install/hadoop-cluster/$installScript root@$HOSTNAME_FQDN:~ ||  { log.error "Unable to upload $SCRIPT_DIR/install/hadoop-cluster/$installScript to $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); return 1; }
		ssh -ttq -n root@$HOSTNAME_FQDN "chmod u+x ${installScript};./${installScript} ${ambariHost} ${AMBARI_VERSION}; rm $installScript" ||  { log.error "Add cluster members step failed." && echo; error_count=$((error_count + 1)); return 1; }
	  fi
	done < $HOSTNAME_FILE
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function createCluster () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.createCluster.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping createCluster step. `cat ${lockfile}`" && return $error_count; fi

	# Create cluster
	echo
	INSTALL_STEP="Create Cluster"
	log.info " ========================================================================"
	log.info " Create Cluster on `hostname -f`"
	log.info " ========================================================================"
	log.info " Cluster to be installed as follows:"
	log.info ""

	ambariHost=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET NODE_SERVER AMBARI" | tr -d '[],')
	ENV_TYPE=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET TOPOLOGY TYPE")
	
	log.info "		Ambari server: 			$ambariHost"
	log.info "		Topology Type: 			$ENV_TYPE"

	log.info ""
	log.info " ========================================================================"

	# Additional properties
	installScript=create.cluster.sh

	# Run Hadoop Cluster installation
	log.info "Starting cluster installation."
	sudo chmod u+x ${SCRIPT_DIR}/install/hadoop-cluster/${installScript} ||  { log.error "Unable to update permissions on ${SCRIPT_DIR}/install/hadoop-cluster/${installScript}." && echo; error_count=$((error_count + 1)); return 1; }
	sudo ${SCRIPT_DIR}/install/hadoop-cluster/$installScript $ENV_TYPE $HDP_VERSION $ambariHost ||  { log.error "Cluster creation failed." && echo; error_count=$((error_count + 1)); return 1; }

	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}


function deployFCILicense () {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.deployFCILicense.lock"
	INSTALL_STEP="Deploy License files"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping deployFCILicense step. `cat ${lockfile}`" && return $error_count; fi
	# Deploy license files
	log.info "Deploy software license files to servers."
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
	  if [[ $HOST_TYPE == hadoop* ]] ; then
		log.info "Copy software license files to /opt/ibm/fci/license/ on $HOSTNAME_FQDN."
		scp $SCRIPT_DIR/license/${LICENSE}.${LICENSE_FORMAT} root@$HOSTNAME_FQDN:~ ||  { log.error "Unable to upload $SCRIPT_DIR/license/${LICENSE}.${LICENSE_FORMAT} license file." && echo; error_count=$((error_count + 1)); }
		ssh -q -n root@$HOSTNAME_FQDN "mkdir -p /opt/ibm/fci/license/; unzip -q -o ${LICENSE}.${LICENSE_FORMAT} -d /opt/ibm/fci/license/; rm ${LICENSE}.${LICENSE_FORMAT}" ||  { log.error "Unable to extract license file." && echo; error_count=$((error_count + 1)); }
	  fi
	  if [[ $HOST_TYPE == ambari ]] && [[ $HOSTNAME_FQDN == `hostname -f` ]] ; then
		log.info "Copy software license files to /opt/ibm/fci/license/ on $HOSTNAME_FQDN."
		mkdir -p /opt/ibm/fci/license/ ||  { log.error "Unable to create /opt/ibm/fci/license/." && echo; error_count=$((error_count + 1)); }
		unzip -q -o $SCRIPT_DIR/license/${LICENSE}.${LICENSE_FORMAT} -d /opt/ibm/fci/license  > /dev/null ||  { log.error "Unable to extract ${LICENSE}.${LICENSE_FORMAT} to /opt/ibm/fci/license/." && echo; error_count=$((error_count + 1)); }
	  fi
	done < $HOSTNAME_FILE
	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function clusterCreationComplete () {
	# Create lockfile for successful script completion
	lockfile="${LOCK_DIR}/.${VERSION}.create.cluster.complete.lock"
	if [[ -f "${lockfile}" ]]; then log.info "Cluster configuration done. `cat ${lockfile}`" && exit $error_count; fi

	# Get values for the Ambari server
	while IFS=$' \t' read HOST_TYPE IP_ADDRESS HOSTNAME_FQDN _
	do
		if [[ $HOST_TYPE == ambari ]] && [[ $HOSTNAME_FQDN == `hostname -f` ]] ; then
			ambariHost=$HOSTNAME_FQDN
		fi
	done < $HOSTNAME_FILE
	echo
	INSTALL_STEP=""
	log.info " ========================================================================"
	log.info ""
	log.info "  To Log into the cluster manager:"
	log.info ""
	log.info " 		https://$ambariHost:8081/"
	log.info " 		USER: admin PASSWORD: admin"
	log.info ""
	log.info " ========================================================================"
	echo

	# If there are no errors, create a lockfile to mark completion
	if [[ $rc = 0 ]]; then
		echo "Completed `date`">$lockfile;
		INSTALL_STEP="CREATE CLUSTER COMPLETE"
		log.info "The installation of the Hadoop Cluster has completed successfully"
	fi
}


function protectPasswords() {

    INSTALL_STEP="protectPasswords"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"

    # actually, never skip this step
	#if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi

														#---------------------------------------
														# protect Kafka server properties file
														#---------------------------------------
    kafka_info=$(\
	  curl -k -u admin:admin -XGET 'https://localhost:8081/api/v1/clusters/fcicluster/services/KAFKA/components/KAFKA_BROKER'\
    )
    kafka_host=$(\
      $SCRIPT_DIR/config/hadoop-cluster/get_json_from_string.py "$kafka_info" "host_components[0].HostRoles.host_name" "true"\
    )

	log.info "$INSTALL_STEP: Modifying permissions to Kafka server properties file on $kafka_host." && echo

	kafkaServerProperties=/usr/hdp/current/kafka-broker/conf/server.properties

	ssh root@$kafka_host "chmod go-rwx $kafkaServerProperties" \
      || { log.error "$INSTALL_STEP failure: $kafkaServerProperties." \
	  && echo; error_count=$((error_count + 1)); return 1; }

														#---------------------------------------
														# did the whole step work?
														#---------------------------------------
	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function createKafkaTopics() {
    INSTALL_STEP="createKafkaTopics"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi

	log.info "Creating Kafka topics"

    kafka_info=$(\
	  curl -k -u admin:admin -XGET 'https://localhost:8081/api/v1/clusters/fcicluster/services/KAFKA/components/KAFKA_BROKER'\
    )
    kafka_host=$(\
      $SCRIPT_DIR/config/hadoop-cluster/get_json_from_string.py "$kafka_info" "host_components[0].HostRoles.host_name" "true"\
    )

	log.info "$INSTALL_STEP: uploading required files to $kafka_host." && echo
	FCI_INSTALL_DIR=/opt/ibm/fci/install
    FCI_HADOOP_CONF_DIR=$FCI_INSTALL_DIR/config/hadoop-cluster
	installScript=create-kafka-topics.sh
	scp $SCRIPT_DIR/config/hadoop-cluster/kafka/$installScript root@$kafka_host:$FCI_HADOOP_CONF_DIR ||  { log.error "Unable to copy $FCI_HADOOP_CONF_DIR/$installScript to $kafka_host." && echo; error_count=$((error_count + 1)); }
	ssh root@$kafka_host "chmod u+x $FCI_HADOOP_CONF_DIR/$installScript;$FCI_HADOOP_CONF_DIR/$installScript" ||  { log.error "$INSTALL_STEP failed." && echo; error_count=$((error_count + 1)); }

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function configRangerKmsCommonName() {
    INSTALL_STEP="configRangerKmsCommonName"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi
	# Deploy Ranger SSL cert
	log.info "Configuring Ranger KMS common name"
	RANGER_KMS_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_KMS_SERVER")

	log.info "Configuring Ranger KMS common name on $RANGER_KMS_SERVER."
	log.info "Uploading required files to $RANGER_KMS_SERVER." && echo

	installScript=update-ranger-kms-repo.sh

    scp $SCRIPT_DIR/config/hadoop-cluster/ranger/$installScript root@$RANGER_KMS_SERVER:~ ||  { log.error "Unable to upload $installScript to $RANGER_KMS_SERVER." && echo; error_count=$((error_count + 1)); return 1; }
	ssh -ttq -n root@$RANGER_KMS_SERVER "chmod u+x ${installScript};./${installScript}; rm $installScript" ||  { log.error "Configuring Ranger KMS common name failed." && echo; error_count=$((error_count + 1)); return 1; }

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function makeRangerSSLCert () {
    INSTALL_STEP="makeRangerSSLCert"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi
	# Deploy Ranger SSL cert
	log.info "Deploy SSL keystore to Ranger Admin server"
	RANGER_ADMIN_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_ADMIN" | tr -d '[],')

	installScript=make-ranger-ssl-cert.sh

	ENV_TYPE=$(getPropertyValue "${PROPERTY_FILE}" "fci.environment.type")
	CLUSTER_TEMPLATE_NAME=$SCRIPT_DIR/config/hadoop-cluster/fci-hadoop-${ENV_TYPE}-cluster-template.json

	javaTrustPassword=$(getPropertyValue "${PROPERTY_FILE}" "jvm.cacerts.pwd")
	rangerKeyStorePassword=$(grep ranger.service.https.attrib.keystore.pass $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)

	log.info "Uploading required files to $RANGER_ADMIN_SERVER." && echo
    scp $SCRIPT_DIR/config/hadoop-cluster/ranger/$installScript root@$RANGER_ADMIN_SERVER:~ ||  { log.error "Unable to upload $installScript to $RANGER_ADMIN_SERVER." && echo; error_count=$((error_count + 1)); return 1; }

    log.info "Making Ranger keystore on $RANGER_ADMIN_SERVER."
	ssh -ttq -n root@$RANGER_ADMIN_SERVER "chmod u+x ${installScript};./${installScript} ${javaTrustPassword} ${rangerKeyStorePassword}; rm $installScript" ||  { log.error "Make Ranger keystore failed." && echo; error_count=$((error_count + 1)); return 1; }

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}


function deployKafkaKeystores() {
  INSTALL_STEP="deployKafkaKeystores"
  local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"

  # Check if this has completed successfully
  if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi

  log.info "Deploying Kafka keystores"
  KAFKA_KEYS_DIR=/etc/kafka/2.6.4.0-91/0
  KAFKA_KEYS_SYMLINK=/etc/kafka/conf
  KAFKA_SSL_KEYSTORE=$SCRIPT_DIR/config/hadoop-cluster/kafka/kafka.client.keystore.jks
  KAFKA_SSL_TRUSTSTORE=$SCRIPT_DIR/config/hadoop-cluster/kafka/kafka.client.truststore.jks

  for HOSTNAME_FQDN in $(cat $HOSTNAME_FILE | sed -e's/ /\t/g' | tr -s $'\t' | cut -d $'\t' -f3);  do
          log.info "$INSTALL_STEP: Deploying to $HOSTNAME_FQDN."
          if [ $HOSTNAME_FQDN = $(hostname -f) ]; then  # skip ambari
            continue
          fi
          ssh -ttq -n root@$HOSTNAME_FQDN "mkdir -p $KAFKA_KEYS_DIR; ln -sf $KAFKA_KEYS_DIR $KAFKA_KEYS_SYMLINK" ||  { log.error "Unable to create $KAFKA_KEYS_DIR on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
          scp $KAFKA_SSL_KEYSTORE root@$HOSTNAME_FQDN:$KAFKA_KEYS_DIR || { log.error "Unable to copy $KAFKA_SSL_KEYSTORE to $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
          scp $KAFKA_SSL_TRUSTSTORE root@$HOSTNAME_FQDN:$KAFKA_KEYS_DIR || { log.error "Unable to copy $KAFKA_SSL_TRUSTSTORE to $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
          ssh root@$HOSTNAME_FQDN "groupadd hadoop; adduser kafka -g hadoop; chmod 740 $KAFKA_KEYS_DIR/kafka.client.*.jks && chown kafka:hadoop $KAFKA_KEYS_DIR/kafka.client.*.jks" \
          || { log.error "Unable to set Kafka keystore ownership or permissions on $HOSTNAME_FQDN." && echo; error_count=$((error_count + 1)); }
  done

  echo
  # If there are no errors, create a lockfile to mark completion
  if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
  return $error_count
}



function deployFciFiles() {
  INSTALL_STEP="deployFciFiles"
  local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"

  # Check if this has completed successfully
  if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi

  log.info "Deploying FCI files to Hadoop nodes"
  
  # Install FCI core assets

  FCI_UTILS_JAR=fci-core-utils-1.0.3.jar
  FCI_UTILS_LINK=fci-core-utils.jar
  KAFKA_MSG_P12=fci_kafka_msg.p12
  FCI_DIR=/opt/ibm/fci
  FCI_LIB_DIR=$FCI_DIR/lib
  
  FCI_INSTALL_DIR=/opt/ibm/fci/install
  FCI_HADOOP_CONF_DIR=$FCI_INSTALL_DIR/config/hadoop-cluster
  
  fci_install_properties=$FCI_INSTALL_DIR/fci-hadoop.properties
  fci_host_properties=$FCI_INSTALL_DIR/fci-hadoop.*.hosts.properties

  for HOSTNAME_FQDN in $(cat $HOSTNAME_FILE | sed -e's/ /\t/g' | tr -s $'\t' | cut -d $'\t' -f3);  do
	
          log.info "$INSTALL_STEP: Deploying to $HOSTNAME_FQDN."
          
          if [ $HOSTNAME_FQDN = $(hostname -f) ]; then  # skip ambari
            continue
          fi

          ssh root@$HOSTNAME_FQDN "mkdir -p $FCI_HADOOP_CONF_DIR $FCI_LIB_DIR" \
          ||  { log.error "Unable to make FCI directories in $HOSTNAME_FQDN." \
        	&& echo; error_count=$((error_count + 1)); }
        	
          scp $fci_install_properties root@$HOSTNAME_FQDN:$FCI_INSTALL_DIR \
          || { log.error "Unable to deploy $fci_install_properties to $HOSTNAME_FQDN." \
        	&& echo; error_count=$((error_count + 1)); }
         	
          scp $fci_host_properties root@$HOSTNAME_FQDN:$FCI_INSTALL_DIR \
          || { log.error "Unable to deploy $fci_host_properties to $HOSTNAME_FQDN." \
        	&& echo; error_count=$((error_count + 1)); }
 
		scp $SCRIPT_DIR/config/hadoop-cluster/${FCI_UTILS_JAR} root@$HOSTNAME_FQDN:$FCI_LIB_DIR \
		||  { log.error "Unable to upload $SCRIPT_DIR/config/hadoop-cluster/${FCI_UTILS_JAR} to $HOSTNAME_FQDN." \
		&& echo; error_count=$((error_count + 1)); }
		
		scp $SCRIPT_DIR/config/hadoop-cluster/${KAFKA_MSG_P12} root@$HOSTNAME_FQDN:$FCI_LIB_DIR \
		||  { log.error "Unable to upload $SCRIPT_DIR/config/hadoop-cluster/${KAFKA_MSG_P12} to $HOSTNAME_FQDN." \
		&& echo; error_count=$((error_count + 1)); }
		
		ssh root@$HOSTNAME_FQDN "ln -sf $FCI_LIB_DIR/$FCI_UTILS_JAR $FCI_LIB_DIR/$FCI_UTILS_LINK" \
          ||  { log.error "Unable to make the FCI core utils symbolic link on $HOSTNAME_FQDN." \
        	&& echo; error_count=$((error_count + 1)); }
        			
		ssh root@$HOSTNAME_FQDN "groupadd hadoop; useradd fci -g hadoop; chmod -R 750 $FCI_DIR && chown -R fci:hadoop $FCI_DIR " \
          ||  { log.error "Unable to set $FCI_DIR permissions on $HOSTNAME_FQDN." \
        	&& echo; error_count=$((error_count + 1)); }	

  done

  echo
  # If there are no errors, create a lockfile to mark completion
  if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
  return $error_count
}



function configRangerUsersyncTrust() {
	INSTALL_STEP="configRangerUsersyncTrust"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"
	
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi
	
	# configure Ranger Usersync trust
	log.info "Configuring SSL mutual authentication for Ranger Usersync"
	RANGER_USERSYNC_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_USERSYNC" | tr -d '[],')

	installScript=config-ranger-plugin-trust.sh
	
	ENV_TYPE=$(getPropertyValue "${PROPERTY_FILE}" "fci.environment.type")
	CLUSTER_TEMPLATE_NAME=$SCRIPT_DIR/config/hadoop-cluster/fci-hadoop-${ENV_TYPE}-cluster-template.json
	
	javaTrustPassword=$(getPropertyValue "${PROPERTY_FILE}" "jvm.cacerts.pwd")
	rangerKeyStorePassword=$(grep 'ranger.service.https.attrib.keystore.pass' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	ugsyncKeyStorePassword=$(grep 'ranger.usersync.keystore.password' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	ugsyncTrustStorePassword=$(grep 'ranger.usersync.truststore.password' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	
	log.info "Uploading required files to $RANGER_USERSYNC_SERVER." && echo
	scp $SCRIPT_DIR/config/hadoop-cluster/ranger/$installScript root@$RANGER_USERSYNC_SERVER:~ ||  { log.error "Unable to upload $installScript to $RANGER_USERSYNC_SERVER." && echo; error_count=$((error_count + 1)); return 1; }
	
	log.info "Configuring Ranger Usersync trust on $RANGER_USERSYNC_SERVER."
	ssh -ttq -n root@$RANGER_USERSYNC_SERVER " \
		chmod u+x ${installScript}; \
		./${installScript} \
			$javaTrustPassword \
			$rangerKeyStorePassword \
			rangerugsync \
			/etc/ranger-usersync/2.6.4.0-91/0 \
			unixauthservice.jks \
			$ugsyncKeyStorePassword \
			mytruststore.jks \
			$ugsyncTrustStorePassword; \
		rm $installScript" ||  { log.error "Configure Ranger Usersync failed." && echo; error_count=$((error_count + 1)); return 1; }

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function configRangerKmsTrust() {
	INSTALL_STEP="configRangerKmsTrust"
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.$INSTALL_STEP.lock"
	
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the $INSTALL_STEP step. `cat ${lockfile}`" && return $error_count; fi
	
	# configure Ranger KMS trust
	log.info "Configuring SSL mutual authentication for Ranger KMS"
	RANGER_KMS_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_KMS_SERVER" | tr -d '[],')

	installScript=config-ranger-plugin-trust.sh
	
	ENV_TYPE=$(getPropertyValue "${PROPERTY_FILE}" "fci.environment.type")
	CLUSTER_TEMPLATE_NAME=$SCRIPT_DIR/config/hadoop-cluster/fci-hadoop-${ENV_TYPE}-cluster-template.json
	
	javaTrustPassword=$(getPropertyValue "${PROPERTY_FILE}" "jvm.cacerts.pwd")
	rangerKeyStorePassword=$(grep 'ranger.service.https.attrib.keystore.pass' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	
	kmsKeyStorePassword=$(grep 'xasecure.policymgr.clientssl.keystore.password' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	kmsTrustStorePassword=$(grep 'xasecure.policymgr.clientssl.truststore.password' $CLUSTER_TEMPLATE_NAME | cut -d\" -f4)
	
	log.info "Uploading required files to $RANGER_KMS_SERVER." && echo
	scp $SCRIPT_DIR/config/hadoop-cluster/ranger/$installScript root@$RANGER_KMS_SERVER:~ ||  { log.error "Unable to upload $installScript to $RANGER_KMS_SERVER." && echo; error_count=$((error_count + 1)); return 1; }
	
	log.info "Configuring Ranger KMS trust on $HOSTNAME_FQDN."
	ssh -ttq -n root@$RANGER_KMS_SERVER " \
		chmod u+x ${installScript}; \
		./${installScript} \
			$javaTrustPassword \
			$rangerKeyStorePassword \
			rangerkmsagent \
			/etc/ranger-kms/2.6.4.0-91/0 \
			ranger-plugin-keystore.jks \
			$kmsKeyStorePassword \
			ranger-plugin-truststore.jks\
			$kmsTrustStorePassword; \
		rm $installScript" ||  { log.error "Configure Ranger KMS trust failed." && echo; error_count=$((error_count + 1)); return 1; }

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

function restartRangerServices() {
	local error_count=0 lockfile="${LOCK_DIR}/.${VERSION}.restartRangerServices.lock"
	INSTALL_STEP="restartRangerServices"
	
	# Check if this has completed successfully
	if [[ -f "${lockfile}" ]]; then log.info "Skipping the restartRangerServices step. `cat ${lockfile}`" && return $error_count; fi
	
	# Restart Ranger services
	log.info "Restarting Ranger services"
	RANGER_ADMIN_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_ADMIN" | tr -d '[],')
	RANGER_USERSYNC_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_USERSYNC" | tr -d '[],')
	RANGER_KMS_SERVER=$(python ${SCRIPT_DIR}/${READ_FCI_CLUSTER_PY} "GET SERVICE_HOST RANGER_KMS_SERVER" | tr -d '[],')

	log.info "Restarting Ranger Admin on $RANGER_ADMIN_SERVER."
	ssh -ttq -n root@$RANGER_ADMIN_SERVER "/usr/bin/ranger-admin restart" ||  { log.error "Restarting Ranger Admin failed." && echo; error_count=$((error_count + 1)); return 1; }
	sleep 5

	log.info "Restarting Ranger Usersync on $RANGER_USERSYNC_SERVER."
	ssh -ttq -n root@$RANGER_USERSYNC_SERVER "/usr/bin/ranger-usersync restart" ||  { log.error "Restarting Ranger Usersync failed." && echo; error_count=$((error_count + 1)); return 1; }
	sleep 5

	log.info "Restarting Ranger KMS on $RANGER_KMS_SERVER."
	ssh -ttq -n root@$RANGER_KMS_SERVER "/usr/bin/ranger-kms restart" ||  { log.error "Restarting Ranger KMS failed." && echo; error_count=$((error_count + 1)); return 1; }
	sleep 5

	echo
	# If there are no errors, create a lockfile to mark completion
	if [[ $error_count = 0 ]]; then echo "Completed `date`">$lockfile; fi
	return $error_count
}

# End of function definitions
##################################################################
#                                                                #
# Main                                                           #
#                                                                #
##################################################################

INSTALL_STEP="Install Hadoop Cluster"

promptForLicenseIntructions

# Accept license
acceptLicenseAgreement_FCI || { log.error "Cannot install without license agreement acceptance." && echo; exit 1; }

# Determine environment size
determineHostPropertiesFile || { log.error "Unable to determine which properties file to use for installation." && echo; exit 1; }

# Validate host types
validateHostTypes || { log.error "Validation of the host type labels failed. Please update the ${HOSTNAME_FILE} file." && echo; exit 1; }

# Create the Python object used to get information from the properties files.
createFciClusterObject || { log.error "Unable to create the FCI Cluster Python object." && echo; exit 1; }

# Check that required files exist
precheckFiles || { log.error "Required pre-requisite files are missing. Please check that all required files are available." && echo; exit 1; }

# Run a Ping test
pingTest ||  { log.error "Ping test failed. Validate the network configuration and check the ${HOSTNAME_FILE} file for accuracy." && echo; exit 1; }

# Install expect package (for setting up SSH trusts)
installExpect ||  { log.error "Unable to install expect rpm.  Please check that yum is properly configured and the required repositories are enabled." && echo; exit 1; }

# Setup SSH trusts
setupSSHTrust ||  { log.error "Unable to setup SSH Trusts. Verify that ssh is installed on all servers." && echo; exit 1; }

# Run an SSH connection test to verify public key exchange - Check if ssh root trusts are enabled (servers listed in fci-hadoop-*.hosts.properties)
sshTest || { log.error "SSH test to verify SSH key exchange has failed." && echo; rm -f ${LOCK_DIR}/.${VERSION}.setupSSHTrust.lock; exit 1; }

# Check that required MySQL rpms are installed
precheckMySQL || { log.warn "Unable to continue installation due to missing pre-requisite files." && echo; exit 1; }

# Run an SSH connection test to verify public key exchange - Check if ssh root trusts are enabled (servers listed in fci-hadoop-*.hosts.properties)
checkUMASK || { log.error "Please update umask setting before preceding." && echo; exit 1; }

# Update OS config
updateOSconfig || { log.error "Unable to complete the updating of the OS configuration." && echo; exit 1; }

# Install OpenJDK 1.8
installJDK || { log.error "Unable to install Open JDK 1.8." && echo; exit 1; }

# Prepare Ambari SSL configuration
configureAmbariSSL || { log.error "Unable to configure Ambari SSL." && echo; exit 1; }

# Install HDP 2.6.4
installHDP || { log.error "Unable to install HDP 2.6.4." && echo; exit 1; }

# Install agents on cluster members
deployAmbariAgents || { log.error "Unable to deploy Ambari agents on all cluster members." && echo; exit 1; }

# Make the Ranger SSL cert
makeRangerSSLCert || { log.error "Unable to make the Ranger SSL certficate." && echo; exit 1; }

# Configure SSL mutual auth for Ranger Usersync
configRangerUsersyncTrust || { log.error "Unable to configure SSL mutual authentication for Ranger Usersync" && echo; exit 1; }

# Configure SSL mutual auth for Ranger KMS
configRangerKmsTrust || { log.error "Unable to configure SSL mutual authentication for Ranger KMS" && echo; exit 1; }

# Deploy Kafka keystores
deployKafkaKeystores || { log.error "Unable to deploy Kafka keystores." && echo; exit 1; }

# Deploy other needed installer files e.g. properties ... we should enable yum install FCI from other nodes :(
deployFciFiles || { log.error "Unable to deploy FCI properties to Hadoop nodes." && echo; exit 1; }

# Create FCI Hadoop cluster
createCluster || { log.error "Unable to create FCI hadoop cluster." && echo; exit 1; }

# Configure Ranger KMS commonNameForCertificate
configRangerKmsCommonName || { log.error "Unable to configure the Ranger KMS common name" && echo; exit 1; }

# Restart Ranger services
restartRangerServices || { log.error "Unable to restart Ranger services" && echo; exit 1; }

# Create Kafka Topics
createKafkaTopics || { log.error "Unable to create Kafka topics" && echo; exit 1; }

# Deploy license files to hadoop servers
deployFCILicense || { log.error "Unable to deploy license files." && echo; exit 1; }

# Protect files with passwords
protectPasswords || { log.error "Failed to restrict access to password files." && echo; exit 1; }

# Cluster creation complete message
clusterCreationComplete || { log.warn "Unable to create final create cluster complete lock file." && echo; exit 1; }

# Remove lock on script.
_remove_lock

# Exit
exit $error_count
