#!/bin/sh
##################################################################
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
#
# (C) Copyright IBM Corp. 2017
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
##################################################################
# Timestamp
timestamp () {
	date +"%F_%T"
}

# Capture the current working directory
SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Set aliases
alias log.info='echo [$(timestamp)][INFO][line $LINENO] ${INSTALL_STEP} - '
alias log.error='echo [$(timestamp)][ERROR][line $LINENO] ${INSTALL_STEP} - '
alias log.warn='echo [$(timestamp)][WARNING][line $LINENO] ${INSTALL_STEP} - '

# Input variables
INSTALLER_DIR=$1
HOSTNAME_FILE=$2

INSTALL_STEP="Configure Ambari SSL"

# Error checking
[[ -z "${INSTALLER_DIR}" || ! -d "${INSTALLER_DIR}" ]]	&& log.warn "Install directory $INSTALLER_DIR does not seem to exist or variable did not load."

# Configure Ambari to use https via an openssl CA and ambari front-end as its end-entity/server/leaf
log.info "Configure Ambari to use https via an openssl CA "

# Prep CA database
log.info "Create Ambari server keys directory."
mkdir -p /var/lib/ambari-server/keys || { log.error "Unable to create Ambari server keys directory." && echo; exit 1; }
cd /var/lib/ambari-server

# Back up existing keys
log.info "Back up existing keys."
P=/var/lib/ambari-server/keys ; if [[ -d "${P}" ]]; then mv $P $P/../keys.bkp.$RANDOM ; mkdir -p $P ; cd $P ; fi

# Create new structure
log.info "Create new /var/lib/ambari-server/keys directory structure."
cp /etc/pki/tls/openssl.cnf /var/lib/ambari-server/keys || { log.error "Unable to copy openssl configuration to Ambari server keys directory." && echo; exit 1; }
mkdir -p {newcerts,certs,req,private} || { log.error "Unable to create new Ambari SSL directory structure." && echo; exit 1; }
touch index.pwd || { log.error "Unable to create /var/lib/ambari-server/keys/index.pwd." && echo; exit 1; }
echo 1234 > crlnumber || { log.error "Unable to create /var/lib/ambari-server/keys/crlnumber." && echo; exit 1; }

# Use PKIX recommendation
log.info "Update openssl configuration with PKIX recommendation."
sed -i "s/#basicConstraints = critical,CA:true/basicConstraints = critical,CA:true/g" openssl.cnf || { log.error "Error updating /var/lib/ambari-server/keys/openssl.cnf." && echo; exit 1; }
sed -i "s/basicConstraints = CA:true/# basicConstraints = CA:true/g" openssl.cnf || { log.error "Error updating /var/lib/ambari-server/keys/openssl.cnf." && echo; exit 1; }

# Create CA private key
log.info "Create CA private key." && echo
RANDFILE=$HOME/.rnd
openssl rand -base64 32 > private/ca.pwd || { log.error "Error creating /var/lib/ambari-server/keys/private/ca.pwd." && echo; exit 1; }
openssl genrsa -aes256 -passout file:private/ca.pwd -out private/ca.pem 4096 || { log.error "Error generating /var/lib/ambari-server/keys/private/ca.pem." && echo; exit 1; }

# Self-sign it
echo && log.info "Self-signing private key."
openssl req -config openssl.cnf -extensions v3_ca -sha256 -new -x509 -days 3650 -key private/ca.pem -passin file:private/ca.pwd -out certs/ca.pem -subj "/C=US/ST=Texas/L=Austin/O=IBM/OU=FCI/CN=ca.$(hostname -d)" || { log.error "Error self-signing private key." && echo; exit 1; }

# Create Ambari (leaf/web server) CSR
log.info "Creating Ambari (leaf/web server) CSR." && echo
openssl rand -base64 32 > private/ambari.pwd || { log.error "Error creating /var/lib/ambari-server/keys/private/ambari.pwd." && echo; exit 1; }
openssl req -days 3650 -newkey rsa:2048 -keyout private/ambari.pem -sha256 -passout file:private/ambari.pwd -out req/ambari.pem -outform PEM -subj "/C=US/ST=Texas/L=Austin/O=IBM/OU=FCI/CN=$(hostname -f)" || { log.error "Error creating /var/lib/ambari-server/keys/private/ambari.pem." && echo; exit 1; }

# Sign CSR
echo && log.info "Signing CSR." && echo
openssl x509 -req -days 3650 -in req/ambari.pem -passin file:private/ca.pwd -out certs/ambari.pem -extfile openssl.cnf -extensions v3_req -CA certs/ca.pem -CAkey private/ca.pem -CAcreateserial || { log.error "Error signing CSR." && echo; exit 1; }
# Create jkcs
echo && log.info "Creating jkcs."
openssl rand -base64 32 > private/pkcs12.pwd || { log.error "Error creating /var/lib/ambari-server/keys/private/pkcs12.pwd." && echo; exit 1; }
cat certs/ambari.pem certs/ca.pem > certs/chain.pem || { log.error "Error creating /var/lib/ambari-server/keys/certs/chain.pem." && echo; exit 1; }

# In Ambari, keystore password MUST match CA password -- possible iop limitation/bug
log.info "Creating keystore."
openssl pkcs12 -export -in certs/chain.pem -inkey private/ambari.pem -passin file:private/ambari.pwd -passout pass:$(cat private/ambari.pwd) -out keystore.p12 || { log.error "Error creating keystore.p12." && echo; exit 1; }

# Create symlinks that Ambari expects
log.info "Creating symbolic links."
ln -s private/ambari.pem ca.key || { log.error "Error creating ca.key symbolic link." && echo; exit 1; }
ln -s certs/ambari.pem ca.crt || { log.error "Error creating ca.crt symbolic link." && echo; exit 1; }
ln -s openssl.cnf ca.config || { log.error "Error creating ca.config symbolic link." && echo; exit 1; }
ln -s private/ambari.pwd pass.txt || { log.error "Error creating pass.txt symbolic link." && echo; exit 1; }
ln -s req/ambari.pem ca.csr || { log.error "Error creating ca.csr symbolic link." && echo; exit 1; }

# Create symlinks that Ambari web server expects, AKA api.ssl (see ambari-server setup-security > https)
ln -s ca.key https.key || { log.error "Error creating https.key symbolic link." && echo; exit 1; }
ln -s ca.crt https.crt || { log.error "Error creating https.crt symbolic link." && echo; exit 1; }
ln -s keystore.p12 https.keystore.p12 || { log.error "Error creating https.keystore.p12 symbolic link." && echo; exit 1; }
ln -s pass.txt https.pass.txt || { log.error "Error creating https.pass.txt symbolic link." && echo; exit 1; }

# Finish creating openssl default structure
log.info "Creating openssl default structure."
mkdir -p db || { log.error "Error creating /var/lib/ambari-server/keys/db." && echo; exit 1; }
touch db/index.pwd || { log.error "Error creating /var/lib/ambari-server/keys/db/index.pwd." && echo; exit 1; }
touch db/serial || { log.error "Error creating /var/lib/ambari-server/keys/db/serial." && echo; exit 1; }
mkdir -p db/newcerts || { log.error "Error creating /var/lib/ambari-server/keys/db/newcerts." && echo; exit 1; }

# Read keystore
log.info "Reading keystore." && echo
openssl pkcs12 -in keystore.p12 -passin file:pass.txt -nokeys -nodes || { log.error "Error reading keystore." && echo; exit 1; }
echo

# Make this host trust itself
log.info "Trusting this host"
cat ~/.ssh/id_rsa.pub >> ~/.ssh/authorized_keys || { log.error "Error configuring ssh trust from `hostname -f` to `hostname -f`." && echo; exit 1; }

# Transfer CA crt
log.info "Transferring CA certificate to each host's trust store: /etc/pki/ca-trust/source/anchors/"
cat $INSTALLER_DIR/$HOSTNAME_FILE | awk 'system("scp certs/ca.pem root@" $3 ":/etc/pki/ca-trust/source/anchors/ca.crt")' || { log.error "Error updating trust stores." && echo; exit 1; }

# Update-ca-trust
log.info "Running up-ca-trust extract on each host"
cat $INSTALLER_DIR/$HOSTNAME_FILE | awk 'system("ssh root@" $3 " \"update-ca-trust extract\" ")' || { log.error "Error extracting update-ca-trust." && echo; exit 1; }

# Continue install
echo && cd $INSTALLER_DIR
