#!/bin/sh
##################################################################
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
#
# (C) Copyright IBM Corp. 2017, 2018
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
##################################################################
# Timestamp
timestamp () {
	date +"%F_%T"
}

# Capture the current working directory
SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Input variables
ENV_TYPE=$1				   # Environment Size (dev) small or (prod) medium
HDP_VERSION=$2           # Version of HDP (including build # - i.e. 2.6.4.0-91)
AMBARI_HOSTNAME=$3		   # Ambari hostname

# Set aliases
alias log.info='echo [$(timestamp)][INFO][line $LINENO] ${INSTALL_STEP} - '
alias log.error='echo [$(timestamp)][ERROR][line $LINENO] ${INSTALL_STEP} - '
alias log.warn='echo [$(timestamp)][WARNING][line $LINENO] ${INSTALL_STEP} - '

# Set defaults if not set
[[ "${AMBARI_HOSTNAME}" = "" ]]     && AMBARI_HOSTNAME="`hostname -f`"
[[ "${ENV_TYPE}" = "" ]]     		&& ENV_TYPE="prod"

# Other variables
CLUSTER_NAME=fcicluster
CONFIG_DIR="${SCRIPT_DIR}/../../config/hadoop-cluster/"
VDF_LOCAL=HDP-$HDP_VERSION.local.xml
HDP_VDF_JSON=fci-hadoop-1.0.3-hdp-VDF.json
BLUEPRINT_NAME=fci-${ENV_TYPE}

# Set json config files based on size
CLUSTER_TEMPLATE_NAME=fci-hadoop-${ENV_TYPE}-cluster-template.json
CLUSTER_BLUEPRINT_NAME=fci-hadoop-${ENV_TYPE}-blueprint-hdp-2.6.4.json

##################################################################

INSTALL_STEP="Create Cluster"
# Check for cluster files
log.info "Check for cluster configuration files."
if [[ ! -f "${CONFIG_DIR}/${CLUSTER_TEMPLATE_NAME}" ]]; then log.error "Unable to find ${CONFIG_DIR}/${CLUSTER_TEMPLATE_NAME}." && exit 1; fi
if [[ ! -f "${CONFIG_DIR}/${CLUSTER_BLUEPRINT_NAME}" ]]; then log.error "Unable to find ${CONFIG_DIR}/${CLUSTER_TEMPLATE_NAME}." && exit 1; fi

log.info "Check for version definition file."
if [[ ! -f "/var/www/html/repos/hdp/HDP/centos7/$HDP_VERSION/HDP-$HDP_VERSION.xml" ]]; then log.error "Unable to find /var/www/html/repos/hdp/HDP/centos7/$HDP_VERSION/HDP-$HDP_VERSION.xml." && exit 1; fi

# Update blueprint with HDP version
log.info "Update blueprint with HDP build version."
sed -i -e "s/hdp.build.version/$HDP_VERSION/g" ${CONFIG_DIR}/${CLUSTER_TEMPLATE_NAME}

# Update Version Definition File
log.info "Update local version definition file."
/bin/cp $CONFIG_DIR/$VDF_LOCAL /var/www/html/repos/hdp/HDP/centos7/$HDP_VERSION/
chmod 744 /var/www/html/repos/hdp/HDP/centos7/$HDP_VERSION/$VDF_LOCAL
sed -i -e "s/ambari.server/$AMBARI_HOSTNAME/g" /var/www/html/repos/hdp/HDP/centos7/$HDP_VERSION/$VDF_LOCAL

# Register VDF file
log.info "Register version definition file."
sed -i -e "s/ambari.server/$AMBARI_HOSTNAME/g" $CONFIG_DIR/$HDP_VDF_JSON
sed -i -e "s/hdp.build.version/$HDP_VERSION/g" $CONFIG_DIR/$HDP_VDF_JSON
curl -v -k -u admin:admin -H "X-Requested-By:ambari" -X POST https://$AMBARI_HOSTNAME:8081/api/v1/version_definitions -d@$CONFIG_DIR/$HDP_VDF_JSON

# Display version definitions
log.info "Display version definitions."
curl -v -k -u admin:admin -H "X-Requested-By:ambari" -X GET https://$AMBARI_HOSTNAME:8081/api/v1/version_definitions

# Verify the hosts defined on the Ambari server
log.info "Verifying hostnames."
echo
curl -H "X-Requested-By: ambari" -u admin:admin -X GET https://$AMBARI_HOSTNAME:8081/api/v1/hosts
echo

# Register blueprint with Ambari server
#
#   "Blueprints" : {
#    "stack_name" : "HDP",
#    "stack_version" : "2.6",
#    "blueprint_name" : "fci-1.0.3"
#  }

# Register blueprint
log.info "Register blueprint with Ambari server."
echo
curl -H "X-Requested-By: ambari" -X POST -u admin:admin https://$AMBARI_HOSTNAME:8081/api/v1/blueprints/$BLUEPRINT_NAME -d @${CONFIG_DIR}/${CLUSTER_BLUEPRINT_NAME} ||  { log.error "Unable to register blueprint with Ambari server." && echo; exit 1; }
echo

# Check blueprint registration
log.info "Verify that blueprint is registered with Ambari."
echo
curl -u admin:admin https://$AMBARI_HOSTNAME:8081/api/v1/blueprints/
echo

# Submit cluster creation request
log.info "Run cluster creation."
echo
curl -H "X-Requested-By: ambari" -X POST -u admin:admin https://$AMBARI_HOSTNAME:8081/api/v1/clusters/${CLUSTER_NAME} -d @${CONFIG_DIR}/${CLUSTER_TEMPLATE_NAME} > create.cluster.request
cat create.cluster.request
echo

# Determine cluster creation request URL
create_cluster_url=`cat create.cluster.request | grep "href" | head -n 1 | sed 's/^.*: \"//' | tr -d \",`

# Deployment status note
log.info "Checking cluster creation process."
echo
curl -u admin:admin -s -H "X-Requested-By: ambari" -X GET $create_cluster_url | head -n 24 ||  { log.error "Unable to initiate cluster creation." && echo; exit 1; }
echo
log.info " ========================================================================"
log.info ""
log.info " Cluster creation started....."
log.info ""
log.info " [Optional] Log into the cluster manager to monitor installation progress."
log.info ""
log.info " 		https://$AMBARI_HOSTNAME:8081/"
log.info " 		USER: admin PASSWORD: admin"
log.info ""
log.info " ========================================================================"
echo

# Check deployment status
log.info "Checking cluster creation process status."
create_cluster_status=""
while [ "$create_cluster_status" != "COMPLETED" ];
do
	if [ "$create_cluster_status" = "FAILED" ]; then
		echo && log.error "Terminating.  Unable to create the hadoop cluster. Please check the Ambari logs." && echo
		exit 1
	elif [ "$create_cluster_status" = "ABORTED" ]; then
		echo && log.error "The cluster creation request has aborted. Please check the Ambari logs." && echo
		exit 1
	else
		create_cluster_status=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $create_cluster_url | head -n 24 | grep "request_status" | sed 's/^.*: \"//' | tr -d \",`
		percent_complete=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $create_cluster_url | head -n 24 | grep "progress_percent" | sed 's/^.*: \"//' | tr -d \", | sed 's/^.*: //' | grep -o '^[0-9]*\.[0-9]'`

		log.info "Create cluster status is currently: $create_cluster_status"
    log.info "Percent Complete: ${percent_complete}% - Checking again in 60 seconds..."
		sleep 60s
	fi
done

# Cluster creation complete
echo && log.info "Cluster creation is complete."

INSTALL_STEP="Restart Cluster"
# Stop all cluster services
log.info "Stopping all services in the cluster" && echo
curl -H 'X-Requested-By: ambari' -X PUT -u admin:admin -d '{"RequestInfo": {"context" : "put services into STOPPED STATE"}, "Body": {"ServiceInfo": {"state": "INSTALLED"}}}' https://${AMBARI_HOSTNAME}:8081/api/v1/clusters/${CLUSTER_NAME}/services/ > stop.services.request
cat stop.services.request

# Determine stop cluster request URL
stop_services_url=`cat stop.services.request | grep "href" | head -n 1 | sed 's/^.*: \"//' | tr -d \",`
echo $stop_services_url

# Check stop services status
echo && log.info "Checking cluster stop services request status."
stop_services_status=""
while [ "$stop_services_status" != "COMPLETED" ];
do
	if [ "$stop_services_status" = "FAILED" ]; then
		echo && log.error "Terminating.  Unable to stop cluster. Check the Ambari logs." && echo
		exit 1
	elif [ "$stop_services_status" = "ABORTED" ]; then
		echo && log.error "The cluster stop request has aborted. Please check the Ambari logs." && echo
		exit 1
	else
		stop_services_status=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $stop_services_url | head -n 24 | grep "request_status" | sed 's/^.*: \"//' | tr -d \",`
		percent_complete=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $stop_services_url | head -n 24 | grep "progress_percent" | sed 's/^.*: \"//' | tr -d \", | sed 's/^.*: //' | grep -o '^[0-9]*\.[0-9]'`
		log.info "Stop services request is currently: $stop_services_status"
    log.info "Percent Complete: ${percent_complete}% - Checking again in 20 seconds..."
		sleep 20s
	fi
done

# Stop all cluster services complete
echo && log.info "Stopping of the cluster services is complete."

# Start all cluster services
log.info "Starting all services in the cluster" && echo
curl -H 'X-Requested-By: ambari' -X PUT -u admin:admin -d '{"RequestInfo": {"context" :"put services into STARTED STATE"}, "Body": {"ServiceInfo": {"state": "STARTED"}}}' https://${AMBARI_HOSTNAME}:8081/api/v1/clusters/${CLUSTER_NAME}/services > start.services.request
cat start.services.request

# Determine start cluster request URL
start_services_url=`cat start.services.request | grep "href" | head -n 1 | sed 's/^.*: \"//' | tr -d \",`
echo $start_services_url

# Check start services status
echo && log.info "Checking cluster start services request status."
start_services_status=""
while [ "$start_services_status" != "COMPLETED" ];
do
	if [ "$start_services_status" = "FAILED" ]; then
		echo && log.error "Terminating.  Unable to start cluster. Check the Ambari logs." && echo
		exit 1
	elif [ "$start_services_status" = "ABORTED" ]; then
		echo && log.error "The cluster start request has aborted. Please check the Ambari logs." && echo
		exit 1
	else
		start_services_status=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $start_services_url | head -n 24 | grep "request_status" | sed 's/^.*: \"//' | tr -d \",`
		percent_complete=`curl -N -u admin:admin -s -H "X-Requested-By: ambari" -X GET $start_services_url | head -n 24 | grep "progress_percent" | sed 's/^.*: \"//' | tr -d \", | sed 's/^.*: //' | grep -o '^[0-9]*\.[0-9]'`
		log.info "Start services request status is currently: $start_services_status"
    log.info "Percent Complete: ${percent_complete}% - Checking again in 20 seconds..."
		sleep 20s
	fi
done

# Cleanup
/bin/rm -f create.cluster.request start.services.request stop.services.request

# Start all cluster services complete
echo && log.info "Starting of the cluster services is complete."
exit 0
