#!/bin/sh
##################################################################
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
#
# (C) Copyright IBM Corp. 2017, 2018
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
##################################################################
# Timestamp
timestamp () {
	date +"%F_%T"
}

# Capture the current working directory
SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

# Set aliases
alias log.info='echo [$(timestamp)][INFO][line $LINENO] ${INSTALL_STEP} - [Hortonworks Data Platform] '
alias log.error='echo [$(timestamp)][ERROR][line $LINENO] ${INSTALL_STEP} - [Hortonworks Data Platform] '
alias log.warn='echo [$(timestamp)][WARNING][line $LINENO] ${INSTALL_STEP} - [Hortonworks Data Platform] '

# Input variables
MEDIA_DIR=$1				# Location of HDP media files
AMBARI_UI_PORT=$2		# Ambari UI port - defaults to 8081
PG_ADMIN_PWD=$3			# PostgreSQL administrator password
AMBARI_REPO=$4			# Ambari repository
HDP_REPO=$5					# HDP REPO
HDP_UTILS_REPO=$6		# HDP_UTILS_REPO
HDP_GPL_REPO=$7			# HDP_GPL REPO
MYSQL_LIB=$8				# MySQL jdbc connector
AMBARI_VERSION=$9   # Ambari build version (i.e. 2.6.1.5-3)
HDP_VERSION=${10}   # Version of HDP (including build # - i.e. 2.6.4.0-91)

# Set defaults if not set
[[ "${MEDIA_DIR}" = "" ]]		&& MEDIA_DIR="${SCRIPT_DIR}"
[[ "${AMBARI_UI_PORT}" = "" ]]	&& AMBARI_UI_PORT="8081"

# Define variables
typeset -i rc=0
CONFIG_DIR="${SCRIPT_DIR}/../../config/hadoop-cluster/"
VDF_LOCAL=HDP-$HDP_VERSION.local.xml

##################################################################

function precheckFiles() {
	local rc=0 error_count=0
	# Check for required properties files
	INSTALL_STEP="Checking for required files"
	log.info "=================================================================================="
	log.info "Checking for required repository files in $MEDIA_DIR."
	log.info "=================================================================================="
	if [[ ! -f "${MEDIA_DIR}/$AMBARI_REPO" ]]; then
		log.error "Required repository file: $AMBARI_REPO could not be found in $MEDIA_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $AMBARI_REPO repository file."
	fi
	if [[ ! -f "${MEDIA_DIR}/$HDP_REPO" ]]; then
		log.error "Required repository file: $HDP_REPO could not be found in $MEDIA_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_REPO repository file."
	fi
	if [[ ! -f "${MEDIA_DIR}/$HDP_UTILS_REPO" ]]; then
		log.error "Required repository file: $HDP_UTILS_REPO could not be found in $MEDIA_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_UTILS_REPO repository file."
	fi
	if [[ ! -f "${MEDIA_DIR}/$HDP_GPL_REPO" ]]; then
		log.error "Required repository file: $HDP_GPL_REPO could not be found in $MEDIA_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $HDP_GPL_REPO repository file."
	fi
	if [[ ! -f "${MEDIA_DIR}/$MYSQL_LIB" ]]; then
		log.error "Required repository file: $MYSQL_LIB could not be found in $MEDIA_DIR."
		error_count=$((error_count + 1))
	else
		log.info "Found $MYSQL_LIB library file."
	fi
	if [[ "$error_count" != "0" ]]; then
		log.error "Total number of missing pre-requisite files: $error_count." && echo
		rc=$error_count
		exit 255
	fi
	echo
	return $rc
}


function createAmbariRepo () {

	INSTALL_STEP="Create mirror repository"
	# Create Hortonworks Data Platform mirror repository
	log.info "Installing HTTP Server."
	echo
	# Install HTTP server
	yum -y install httpd || { log.error "Unable to install httpd on `hostname -f`." && echo; exit 1; }
	echo

	log.info "Starting HTTP Server."
	# Enable HTTPD server
	systemctl enable httpd
	# Start HTTP server
	systemctl start httpd || { log.warn "Unable to start httpd service on `hostname -f`." && echo; }

	#Update firewall to allow inbound http
	log.info "Updating firewall to allow inbound HTTP requests."
	echo
	firewall-cmd --zone=public --add-port=80/tcp --permanent
	firewall-cmd --reload
	echo

	# Create repos directory
	log.info "Creating local repository directory."
	mkdir -p /var/www/html/repos/hdp/HDP-UTILS/

	# ----------------------------------
	# Tar balls
	# ----------------------------------
	# FCI 1.0.3
	# ----------------------------------
	# ambari-2.6.1.5-centos7.tar.gz
	# HDP-2.6.4.0-centos7-rpm.tar.gz
	# HDP-UTILS-1.1.0.22-centos7.tar.gz
	#
	# AMBARI
	# http://public-repo-1.hortonworks.com/ambari/centos7/2.x/updates/2.6.1.5/ambari-2.6.1.5-centos7.tar.gz
	#
	# HDP
	# http://public-repo-1.hortonworks.com/HDP/centos7/2.x/updates/2.6.4.0/HDP-2.6.4.0-centos7-rpm.tar.gz
	# http://public-repo-1.hortonworks.com/HDP/centos7/2.x/updates/2.6.4.0/HDP-2.6.4.0-centos7-rpm.tar.gz.md5
	#
	# HDP-UTILS
	# http://public-repo-1.hortonworks.com/HDP-UTILS-1.1.0.22/repos/centos7/HDP-UTILS-1.1.0.22-centos7.tar.gz
	# http://public-repo-1.hortonworks.com/HDP-UTILS-1.1.0.22/repos/centos7/HDP-UTILS-1.1.0.22-centos7.tar.gz.md5
	# ----------------------------------

	# Determine which version of HDP-UTILS was downloaded (HDP has multiple mirrors with multiple packages of the same name)
	md5sum ${MEDIA_DIR}/$HDP_UTILS_REPO > ${MEDIA_DIR}/checksum_hdputils
	if [[ `cat ${MEDIA_DIR}/checksum_hdputils | awk '{ print $1 }'` == "e0b825f1e12835aa9f04ef26ebf7916b" ]]; then
		# The Nov 2017 version of the file
		HDP_UTILS_DIR="/var/www/html/repos/hdp/HDP-UTILS/"
	else
		if  [[ `cat ${MEDIA_DIR}/checksum_hdputils | awk '{ print $1 }'` == "f7f3542361e5d4886e6b816940dd8398" ]]; then
			# The May 2018 version of the file
			HDP_UTILS_DIR="/var/www/html/repos/hdp/"
		else
			HDP_UTILS_DIR="/var/www/html/repos/hdp/"
		fi
		# Check package and adjust VDF file based on package
		chmod 744 $CONFIG_DIR/$VDF_LOCAL
		sed -i -e "s#/repos/hdp/HDP-UTILS/#/repos/hdp/HDP-UTILS/centos7/1.1.0.22/#g" $CONFIG_DIR/$VDF_LOCAL
	fi

	# Cleanup checksum file
	rm -f ${MEDIA_DIR}/checksum_hdputils

	# Extract repos
	log.info "=================================================================================="
	log.info "Extracting Ambari repository."
	tar -xzf ${MEDIA_DIR}/$AMBARI_REPO -C /var/www/html/repos && log.info " Success"
	log.info "=================================================================================="
	log.info "Extracting Hortonworks Data Platform repository."
	tar -xzf ${MEDIA_DIR}/$HDP_REPO -C /var/www/html/repos/hdp && log.info " Success"
	log.info "=================================================================================="
	log.info "Extracting Hortonworks Data Platform utilities repository."
	tar -xzf ${MEDIA_DIR}/$HDP_UTILS_REPO -C $HDP_UTILS_DIR && log.info " Success"
	log.info "=================================================================================="
	log.info "Extracting Hortonworks Data Platform gpl repository."
	tar -xzf ${MEDIA_DIR}/$HDP_GPL_REPO -C /var/www/html/repos/hdp/ && log.info " Success"
	log.info "=================================================================================="

	# Create Ambari repo file in /etc/yum.repos.d
	log.info "Creating Ambari repository in /etc/yum.repos.d/."
	cat << 'EOF' >> /etc/yum.repos.d/ambari.repo
[ambari]
name=Ambari AMBARI_VERSION
baseurl=http://REPO_HOSTNAME/repos/ambari/centos7/AMBARI_VERSION/
gpgcheck=0
enabled=1
priority=1
EOF

	# Add HTTP Server hostname
	log.info "Updating Ambari repository with HTTP server name."
	sed -i -e "s/REPO_HOSTNAME/`hostname -f`/g" /etc/yum.repos.d/ambari.repo

	# Add Ambari version
	log.info "Updating Ambari repository with Ambari version - $AMBARI_VERSION."
	sed -i -e "s/AMBARI_VERSION/$AMBARI_VERSION/g" /etc/yum.repos.d/ambari.repo

	# Install public key
	log.info "Import public key for Ambari rpm packages." && echo
	rpm --import /var/www/html/repos/ambari/centos7/2.6.1.5-3/RPM-GPG-KEY/RPM-GPG-KEY-Jenkins

}

##################################################################

# Check that required files exist
precheckFiles
if [ "$?" -ne "0" ]
then
  log.error "Required pre-requisite files are missing. Please check that all required files are available." && echo
  exit 255
fi

# Create Ambari repository
INSTALL_STEP="Create mirror repository"
log.info "=================================================================================="
log.info "Create local repository for Ambari."
log.info "=================================================================================="
createAmbariRepo

# Install Ambari server
INSTALL_STEP="Install Ambari server"
log.info "=================================================================================="
log.info "Install Ambari server." && echo
yum install -q -y ambari-server ||  { log.error "Installation of Ambari server failed." && echo; exit 1; }
echo

# Update Ambari UI port
log.info "=================================================================================="
log.info "Updating Ambari server configuration."
log.info "=================================================================================="
log.info "Updating Ambari server port number."
echo "client.api.port=${AMBARI_UI_PORT}" >> /etc/ambari-server/conf/ambari.properties

# Make sure Ambari port is open
log.info "Update firewall to allow Ambari SSL port." && echo
firewall-cmd --add-port=${AMBARI_UI_PORT}/tcp --permanent || log.warn "Warning. There was an issue updating the firewall configuration with Ambari port"

# Configure ambari.properties for ssl
echo && log.info "Configuring ambari.properties for SSL (api.ssl)."
cat <<- EOF >> /etc/ambari-server/conf/ambari.properties
	client.api.ssl.key_name=ca.key
	client.api.ssl.cert_name=ca.crt
	client.api.ssl.port=AMBARI_PORT
	api.ssl=true
EOF

# Update Ambari port number in ambari.properties
log.info "Updating Ambari properties with port number."
sed -i -e "s/AMBARI_PORT/${AMBARI_UI_PORT}/g" /etc/ambari-server/conf/ambari.properties

# Changing the default log location to /hadoop/log
log.info "Updating the default log location to /hadoop/log"
mkdir -p /hadoop
sed -i -e "s#/var/log/ambari-server#/hadoop/log/ambari-server#g" /etc/ambari-server/conf/log4j.properties

# Check that Java sdk is installed
log.info "Checking if Open JDK is installed."
if rpm -q java-1.8.0-openjdk-devel > /dev/null; then
	log.info "Package java-1.8.0-openjdk-devel is installed."
else
	log.error "Open JDK 1.8.0 is not installed. Please install java-1.8.0-openjdk-devel on `hostname -f` and restart the installer." && exit 1
fi

# Set the java for the Ambari server
log.info "Updating Ambari server java configuration."
echo
ambari-server setup -j /etc/alternatives/java_sdk_1.8.0_openjdk -s -v ||  { log.error "Unable to update Ambari java configuration." && echo; exit 1; }

# Ambari JDBC configuration - MySQL driver
log.info "Updating Ambari JDBC configuration: MySQL JDBC driver."
mysql_lib_basename=`echo "${MYSQL_LIB%.*.*}"`
mkdir -p $MEDIA_DIR/tmp;tar -zxf $MEDIA_DIR/$MYSQL_LIB -C $MEDIA_DIR/tmp; /bin/cp -rf $MEDIA_DIR/tmp/mysql*/mysql*.jar /usr/share/java/;rm -rf $MEDIA_DIR/tmp
ln -s /usr/share/java/$mysql_lib_basename.jar /usr/share/java/mysql-connector-java.jar
ambari-server setup --jdbc-db=mysql --jdbc-driver=/usr/share/java/mysql-connector-java.jar -s -v ||  { log.error "Unable to update Ambari MySQL JDBC configuration." && echo; exit 1; }
echo

# Ambari JDBC configuration - Postgres driver
log.info "Updating Ambari configuration: Postgres JDBC driver."
ambari-server setup --jdbc-db=postgres --jdbc-driver=/usr/share/java/postgresql-jdbc.jar -s -v ||  { log.error "Unable to update Ambari Postgres JDBC configuration." && echo; exit 1; }
echo

# set Postgres Admin Password
log.info "Setting the PostgreSQL administrator password."
sudo --login -u postgres psql -c "alter user postgres with password '$PG_ADMIN_PWD';" ||  { log.error "Unable to set the postgres administrator password." && echo; exit 1; }

# update postgres host-based authentication configuration
log.info "Updating PostgreSQL host-based authentication for remote JDBC access"
PG_DATA_DIR=/var/lib/pgsql/data
HBA_CFG_FILE=$PG_DATA_DIR/pg_hba.conf
sed -i -e 's#\(^.*local.*all.*postgres\).*$#local all postgres,rangeradmin,rangerlogger,rangerkms trust#' $HBA_CFG_FILE
sed -i -e 's#\(^.*host.*all.*postgres\).*127\.0\.0\.1.*$#host all postgres,rangeradmin,rangerlogger,rangerkms 0.0.0.0/0 trust#' $HBA_CFG_FILE
sed -i -e 's#\(^.*host.*all.*postgres\).*::1/.*$#host all postgres,rangeradmin,rangerlogger,rangerkms ::/0 trust#' $HBA_CFG_FILE
su - postgres -c "pg_ctl reload -D $PG_DATA_DIR"

# reduce flume metrics log level (fci/tribe#1473
log.info "Setting Flume metrics log level"
flume_log4j_template="/var/lib/ambari-server/resources/common-services/FLUME/*/package/templates/log4j.properties.j2"
sed -i\
 -e '/^log4j.logger.org.apache.hadoop.metrics2.sink.flume.FlumeTimelineMetricsSink/d'\
  -e '/^log4j.logger.org.apache.hadoop/a log4j.logger.org.apache.hadoop.metrics2.sink.flume.FlumeTimelineMetricsSink = WARN'\
   $flume_log4j_template || log.warn "Failed to set Flume metrics log level"

log.info "Starting Ambari server." && echo
ambari-server start ||  { log.error "Error starting Ambari server." && echo; exit 1; }
echo
